Option Explicit

'***************************************************************
'*      _______     __                                         *
'*     |  ___\ \   / /                                         *
'*     | |__  \ \_/ /                                          *
'*     |  __|  \   /                      Ernst & Young        *
'*     | |____  | |            Windows Inventory Script        *
'*     |______| |_|                                            *
'*                                                             *
'***************************************************************

'Included features:
'
' Online detection
' Active Directory
' DNS Records
' CPU
' DiskPace
' IP
' Processes
' Services
' Windows Cluster
' Applications
' Event Log
' HyperV
' User Logons
' ISA/Forefront
' Exchange
' BizTalk
' SQL
' SharePoint

'********************************************
Const Version      = "4.0.8"                       ' Do not change
Const ScriptName   = "EYInventoryScript.vbs"       ' Do not change
                                                   '
Const OutputFolder = "EY_Output"                   ' Do not change
Const FilePrefix   = "_EY_"                        ' Do not change
                                                   '
Const ageLimit           = 6500                    ' Max age to be added in DeviceList.txt
Const bOnlyCheckServers  = False                   ' Whether to skip workstations during scan
Const bOnlyServersFromAD = False                   ' Whether to include workstations in DeviceList.txt
Const maxThreadCount     = 10                      '
Const maxCPUload         = 95                      ' Load limit on server being scanned
Const maxCPUloadHost     = 70                      ' Load limit on scan host
Const maxRunningTime     = 4320                    ' Max script run time in minutes
Const maxRunningPasses   = 2                       ' Max scanning attempts
Const minBytesNeeded     = 10                      ' Used for checking for empty output
                                                   '
'**********************************************
Dim args                                           '  
Set args = CreateObject("Scripting.Dictionary")    ' 
                                                   '    
Dim strDomain:   strDomain   = "<notset>"          ' Explicit windows credentials (use commandline options)
Dim strUser:     strUser     = "<notset>"          '
Dim strPassword: strPassword = "<notset>"          '
                                                   '
Dim strUserSQL:     strUserSQL     = ""            ' Explicit SQL credentials (change as needed)
Dim strPasswordSQL: strPasswordSQL = ""            '
                                                   '
Dim strVF: strVF = ""                              ' 
                                                   '
'**********************************************
Dim WMI, WMISQL(5), WMIDNS, WMICLUS, WMIVIRT, REG, FSO, DSE, DNC, WSH
Dim Win2kMode
Dim bLocalMode: bLocalMode = False
Dim bADDomainFound: bADDomainFound = False
Dim FullPath, OutputPath
Dim bIsWorkstation: bIsWorkstation = False

Dim strDomainRole: strDomainRole = ""
Dim strComputer: strComputer = ""
Dim strRealComputerName, strAliasComputerName, strLocalComputerName: strAliasComputerName = ""
Dim strComputerDomain: strComputerDomain = ""

'********************************************
Const HKCU = &H80000001 'HKEY_CURRENT_USER
Const HKLM = &H80000002 'HKEY_LOCAL_MACHINE
Const HKU  = &H80000003 'HKEY_USERS

Const PktPrivacy = 6
Const wbemAuthenticationLevelPkt = 6

'********************************************
'Purpose:
'Initializes script and sets important global variables - also performs WScript version checking
Sub Initialize
	On Error Resume Next
	Err.clear

    ' WScript Shell
	Call ConnectWSH()
	
	' Force CScript
	If Right(UCase(WScript.fullname),11)= "WSCRIPT.EXE" Then
        RestartScriptWithCScript
		WScript.Quit
	End If
	' Script name
	If Not Instr(UCase(WScript.ScriptFullName),UCase(ScriptName)) > 0 Then
        WScript.Echo "The script has been changed from its original value"
        WScript.Echo "Rename to " & UCase(ScriptName) & " and restart the script"
        WScript.Echo "Exiting"
		WScript.Quit
	End If
	' WSH version
	If CInt(Replace(UCase(WScript.Version),".","")) < 56 Then
		Win2kMode = True
	Else
		Win2kMode = False
	End If
	
	' FileSystem
	Call ConnectFSO()
	
	' Set default computer
	Call Computer_LocalName()
	strComputer = strLocalComputerName
	
	' Network paths
	Dim NetPath: If Left(WScript.ScriptFullName, 2) = "\\" Then: NetPath = "\": Else NetPath = "": End If

	' Directory
	If Win2kMode Then
		FullPath = NetPath & Replace(Replace(UCase(WScript.ScriptFullName),UCase(ScriptName),"") & "\","\\","\")
		OutputPath = NetPath & Replace(FullPath & OutputFolder & "\","\\","\")
		' Output folder
		If Not FSO.FolderExists(OutputPath) Then: FSO.CreateFolder(OutputPath): End If
	Else
		FullPath = NetPath & Replace(Replace(UCase(WScript.ScriptFullName),UCase(ScriptName),"") & "\","\\","\")
		WSH.CurrentDirectory = FullPath
		OutputPath = NetPath & Replace(FullPath & OutputFolder & "\","\\","\")
		'Output folder
		If Not FSO.FolderExists(OutputPath) Then: FSO.CreateFolder(OutputPath): End If
		WSH.CurrentDirectory = OutputPath
	End If

End Sub

Sub RestartScriptWithCScript
    Dim strCommandLine: strCommandLine = "CScript.exe /NOLOGO """
    strCommandLine = strCommandLine + WScript.ScriptFullName
    strCommandLine = strCommandLine + """ "

    Dim arg: For Each arg In WScript.Arguments
		strCommandLine = strCommandLine + arg + " "
	Next

    Call WSH.Run(strCommandLine, 1, False)
End Sub

'********************************************
'Purpose:
'Finishes script initialization and sets important global variables
Function Initialize2 (bRunRemoteRegScriptOnFailedWMI)
	On Error Resume Next
	Err.clear
	
	' Initialize variables to defaults - will be overwritten if necessary
	strRealComputerName  = UCase(strComputer)
	strAliasComputerName = UCase(strComputer)
	
	' Windows Management Interface + Registry
	If Not ConnectWMI() Then
        If bRunRemoteRegScriptOnFailedWMI Then
            RunInstalledSoftwarePSScript(strComputer)
        End If

        ' We were unable to connect using WMI - No point in proceeding
        WScript.Echo "*** Unable to connect to " & strComputer & " using WMI ***"
		WScript.Quit
    End If

	' Domain role
	Call Computer_DomainRole()
	
	' Admin access
	Err.clear
	Dim key: Call REG.GetStringValue(HKU, "S-1-5-19\Environment", "TEMP", key)
	If Err <> 0 Then
		Touch("noadmin")
		WScript.Echo "*** The current user account does have Admin access on " & strComputer
		WScript.Quit
	End If
End Function

'********************************************
'Purpose:
'Parses commandline options and executes the appropriate functions
Sub Execute
	On Error Resume Next
	Err.clear
	
    ParseCommandLine()

	If TaskSpecifiedOnCommandLine Then
        ' Execute the chosen task
        ExecuteChosenTask()
    Else
        Do
            ShowAvailableTasks()
            ExecuteChosenTask()

            ' Reset commandline options for next loop
            ParseCommandLine()

            ' Wait a few seconds to allow the user to review output from last task
            WScript.Sleep 3000

        Loop While True
	End If
	
End Sub

'********************************************
'Purpose:
'Display available Tasks on screen
Sub ShowAvailableTasks
	On Error Resume Next

    Dim strChoice, nChoice, nLineCounter

    If Win2kMode Then
		Do
			WScript.Echo "Press 'L' to scan the local computer"
			WScript.Echo "Press 'L', then <Enter> to continue"
			strChoice = UCase(Wscript.StdIn.ReadLine)
		Loop While strChoice <> "L"
		args("/LOCAL") = ""
	Else
        Do
            for nLineCounter = 1 to 25
                WScript.Echo ""
            next
		    WScript.Echo "  #############################################################"
            WScript.Echo "  #       _______     __                                      #"
            WScript.Echo "  #      |  ___\ \   / /                                      #"
            WScript.Echo "  #      | |__  \ \_/ /                                       #"
            WScript.Echo "  #      |  __|  \   /                  Ernst & Young         #"
            WScript.Echo "  #      | |____  | |        Windows Inventory Script         #"
            WScript.Echo "  #      |______| |_|                 (Version " & Version & ")         #"
            WScript.Echo "  #                                                           #"
            WScript.Echo "  #############################################################"
            WScript.Echo "  *                                                           *"
            WScript.Echo "  *   (1) Scan the local computer                             *"
            WScript.Echo "  *   (2) Scan all computers in Active Directory              *"
            WScript.Echo "  *   (3) Scan all computers in DeviceList.txt                *"  
            WScript.Echo "  *   (4) Export license information from Exchange Server     *"
            WScript.Echo "  *   (5) Export license information from Lync Server         *"
            WScript.Echo "  *   (6) Export license information from SharePoint Server   *"
            WScript.Echo "  *   (7) Export license information from Office 365          *"
            WScript.Echo "  *   (8) Export Virtual Machine information from Azure       *"
            WScript.Echo "  *   (9) Export Virtual Machine information from VMware      *"
            WScript.Echo "  *   (x) Exit                                                *"
            WScript.Echo "  *                                                           *"
            WScript.Echo "  *    Choose option (1-9) or 'x' then <Enter> to continue    *"
            WScript.Echo "  *                                                           *"
            WScript.Echo "  *************************************************************"
            strChoice = UCase(Wscript.StdIn.ReadLine)
            If strChoice = "X" Then
                nChoice = 10
            Else
                nChoice = CInt(strChoice)
            End If
		Loop While IsEmpty(nChoice) Or (nChoice < 1 Or nChoice > 10)

		If nChoice = 1 Then
            args("/LOCAL") = ""
        ElseIf nChoice = 2 Then
             args("/AD") = "" 
        ElseIf nChoice = 3 Then
             args("/DEVICELIST") = "" 
        ElseIf nChoice = 4 Then
            args("/EXCHANGE:") = "" 
        ElseIf nChoice = 5 Then
            args("/LYNC:") = "" 
        ElseIf nChoice = 6 Then
            args("/SHAREPOINT:") = ""
        ElseIf nChoice = 7 Then
            args("/OFFICE365") = "" 
        ElseIf nChoice = 8 Then
            args("/AZURE") = "" 
        ElseIf nChoice = 9 Then
            args("/VMWARE:") = ""
        Else
            WScript.Echo "Goodbye"
            WScript.Quit
        End If
	End If

End Sub

'********************************************
'Purpose:
'Executes the task specified on the command line or options screen
Sub ExecuteChosenTask
	On Error Resume Next
	Err.clear
	
	' Check for server name (should be specified on commandline or automatically detected)
	If strComputer = "" Then
        WScript.Echo "No target computer was specified for scanning"
        WScript.Echo "Exiting"
		WScript.Quit
	End If

	If args("/LOCAL") <> 0 Then
		' Check server!
		bLocalMode = True
		Call Execute_Single()
	
	ElseIf args("/SERVER:") <> 0 Then
        ' Computer is explicit
		strComputer = UCase(args("/SERVER:"))
		Call Execute_Single()
	
	ElseIf args("/AD") <> 0 Then
		' Active Directory RootDSE
		Call ConnectDSE()
		' Active Directory Server list
		Call AD_Devices()
        Call Execute_List()
	
	ElseIf args("/DEVICELIST") <> 0 Then
		' DeviceList!
		Call Execute_List()
		
	ElseIf args("/LISTACTIVECOMPUTERS") <> 0 Then
        ' Active Directory RootDSE
	    Call ConnectDSE()
		' Active Directory Server list
		Call AD_Devices()

	ElseIf args("/LISTADUSERSANDGROUPS") <> 0 Or args("/LISTADGROUPSANDUSERS") <> 0 Then
		 ' Active Directory RootDSE
	    Call ConnectDSE()
        ' Active Directory Users/Groups list
		Call AD_Users()
		Call AD_Groups()

	ElseIf args("/LISTADUSERS") <> 0 Then
        ' Active Directory RootDSE
	    Call ConnectDSE()
		' Active Directory Users list
		Call AD_Users()

	ElseIf args("/LISTADGROUPS") <> 0 Then
        ' Active Directory RootDSE
	    Call ConnectDSE()
		' Active Directory Groups list
		Call AD_Groups()

    ElseIf args("/EXCHANGE:") <> 0 Then
		RunExchangePSScript()

    ElseIf args("/LYNC:") <> 0 Then
		RunLyncPSScript()

    ElseIf args("/OFFICE365") <> 0 Then
		RunOffice365QueryPSScript()

    ElseIf args("/AZURE") <> 0 Then
		RunAzureVMQueryPSScript()

    ElseIf args("/VMWARE:") <> 0 Then
		RunVMwareQueryPSScript()

    ElseIf args("/SHAREPOINT:") <> 0 Then
        Dim strSharePointServer: strExchangeServer = args("/SHAREPOINT:")
		Call RunSharePointPSScript(strSharePointServer, False)

	End If

End Sub

'********************************************
'Purpose:
'Parse command line options
Sub ParseCommandLine
	On Error Resume Next
	Err.clear

    args.RemoveAll()
	
	' All recognized commandline options
	args.Add "/DEVICELIST", 0
	args.Add "/LOCAL", 0
	args.Add "/AD", 0
	args.Add "/LISTACTIVECOMPUTERS", 0
	args.Add "/LISTADUSERSANDGROUPS", 0
	args.Add "/LISTADGROUPSANDUSERS", 0
	args.Add "/LISTADGROUPS", 0
	args.Add "/LISTADUSERS", 0
	args.Add "/SERVER:", 0
	args.Add "/USER:", strUser
	args.Add "/PASSWORD:", strPassword
	args.Add "/DOMAIN:", strDomain
    args.Add "/VF:", 0
    args.Add "/EXCHANGE:", 0
    args.Add "/LYNC:", 0
    args.Add "/OFFICE365", 0
    args.Add "/AZURE", 0
    args.Add "/VMWARE:", 0
    args.Add "/SHAREPOINT:", 0
	
	' Parse commandline options and store values in args
	Dim arg: For Each arg In WScript.Arguments
		Dim bFound: bFound = False
		Dim key: For Each key In args
			If (InStr(UCase(arg),key) > 0) And (Len(arg) > 0) Then
				bFound = True
				args(key) = Mid(Trim(arg),Len(key)+1)
				
				If Win2kMode And key <> "/LOCAL" Then
                    WScript.Echo "This script only supports 'localmode' inventory collection on Windows 2000 and older"
                    WScript.Echo "Exiting"
					WScript.Quit
				End If
			End If
		Next
		If Not bFound Then
			WScript.Echo "The specified argument: """ & arg & """ is not supported"
			WScript.Quit
		End If
	Next
	
    ' Set explicit credentials (defaults to <notset>)
    strUser     = args("/USER:")
	strPassword = args("/PASSWORD:")
	strDomain   = args("/DOMAIN:")

    If args("/VF:") <> 0 Then
		strVF = args("/VF:")
	End If

End Sub

'********************************************
'Purpose:
'Was a task specified on the command line
Function TaskSpecifiedOnCommandLine
	On Error Resume Next
	Dim bTaskSpecified: bTaskSpecified = False

    If args("/LOCAL") <> 0 Or _
        args("/SERVER:") <> 0  Or _
        args("/AD") <> 0 Or _
        args("/DEVICELIST") <> 0 Or _
        args("/LISTACTIVECOMPUTERS") <> 0 Or _
        args("/LISTADUSERSANDGROUPS") <> 0 Or _
        args("/LISTADGROUPSANDUSERS") <> 0 Or _
        args("/LISTADUSERS") <> 0 Or _
        args("/LISTADGROUPS") <> 0 Or _
        args("/EXCHANGE:") <> 0 Or _
        args("/LYNC:") <> 0 Or _
        args("/OFFICE365") <> 0 Or _ 
        args("/VMWARE:") <> 0 Or _
        args("/SHAREPOINT:") <> 0 Then 
		bTaskSpecified = True
	End If
	
    TaskSpecifiedOnCommandLine = bTaskSpecified

End Function

'********************************************
'Purpose:
'Scanning of a single host
Sub Execute_Single
	On Error Resume Next
	Err.clear
	
	' Check if server has been scanned already
	If FSO.FileExists(OutputPath & strComputer & FilePrefix & "Done") Then
        WScript.Echo "Computer " & strComputer & " has already been scanned"
        Exit Sub
	End If
	
	' Cleanup
	Call FSO.DeleteFile(OutputPath & strComputer & FilePrefix & "offline", true)
	Call FSO.DeleteFile(OutputPath & strComputer & FilePrefix & "Error", true)
	Call FSO.DeleteFile(OutputPath & strComputer & FilePrefix & "Noadmin", true)
	Call FSO.DeleteFile(OutputPath & strComputer & FilePrefix & "Workstation", true)
	Call FSO.DeleteFile(OutputPath & strComputer & FilePrefix & "LOAD", true)
	Call FSO.DeleteFile(OutputPath & strComputer & FilePrefix & "WMIError", true)
	
	' Check if server is online
    Dim bPingResult: bPingResult = Ping(strComputer)
	If bPingResult Then
		Call Touch("online")
	End If
	
	' Finish initialization
	Call Initialize2(bPingResult)
	
	' Check if server is overloaded
	If CPUload() > maxCPUload Then
		Call Touch("LOAD")
		WScript.Echo "Script CPU load limit has been exceeded"
        Exit Sub
	End If
	
	Dim bSuccess: bSuccess =  True
	
	' Installed software (from registry(WMI))
	If bSuccess Then
		bSuccess = bSuccess And Software()
	End If

	' System information
	If bSuccess Then
		bSuccess = bSuccess And System()
	End If
	
    ' Installed software license info
	If bSuccess Then
		bSuccess = bSuccess And SoftwareLicenseInfo()
	End If

	' Services
	If bSuccess Then
		bSuccess = bSuccess And Services()
	End If

    ' CPU information
	If bSuccess Then
		bSuccess = bSuccess And CPU()
	End If
	
    ' DiskSpace information
		bSuccess = bSuccess And DiskSpace()
	If bSuccess Then
	End If
	
	' System Dates
	If bSuccess Then
		bSuccess = bSuccess And SystemDates()
	End If

    ' Installed Server Features
	If bSuccess Then
		bSuccess = bSuccess And ServerFeatures()
	End If

	' MSIInstaller event log
	If bSuccess Then
		bSuccess = bSuccess And MSIInstaller()
	End If

	' Processes
	If bSuccess Then
		bSuccess = bSuccess And Processes()
	End If

	' Network adapters (IP information)
	If bSuccess Then
		bSuccess = bSuccess And Computer_IP()
	End If

	' User logons on local system
	If bSuccess Then
		bSuccess = bSuccess And Computer_Logons()
	End If

    ' User Profiles on local system
	If bSuccess Then
		bSuccess = bSuccess And UserProfiles()
	End If
    
	If bSuccess Then
		bSuccess = bSuccess And DNS()
	End If

	' HyperV
	If bSuccess Then
		bSuccess = bSuccess And HyperV()
	End If

    ' Acces user groups
    If bSuccess Then
		bSuccess = bSuccess And Computer_Access()
	End If
	
	' Cleanup if not detected
	FileOKOrDelete("SQL_Sharepoint_SiteFeatures.txt")
	FileOKOrDelete("SQL_Sharepoint_Sites.txt")
	FileOKOrDelete("SQL_Sharepoint_Users.txt")
	FileOKOrDelete("SQL_Sharepoint_Config.txt")

	If bSuccess Then
		Touch("Done")
	Else
		Touch("Error")
	End If

End Sub

'********************************************
'Purpose:
'Left padding of a string
Function LPad(str, l)
	On Error Resume Next
	LPad = string(l-Len(str)," ") & str
End Function

'********************************************
'Purpose:
'Determine if a String variable is empty or set to 0
Function IsStringEmptyOrNull(str)
	
    Dim bIsEmptyString
    bIsEmptyString = (str = "") 
    
    If IsEmpty( bIsEmptyString ) Then
        bIsEmptyString = True
    End If

    IsStringEmptyOrNull = bIsEmptyString
End Function

'********************************************
'Purpose:
'Execute a parallelized scan based on a list of devices
Sub Execute_List
	On Error Resume Next
	Err.clear
	
	If Win2kMode Then
		WScript.Echo "This script can only run in 'localmode' on Windows 2000 or older"
		WScript.Echo "If scanning remote computers, use a newer OS."
		Exit Sub
	End If

    WScript.Echo ""
    WScript.Echo " Scanning multiple devices for software & hardware inventory"
    WScript.Echo ""
	
	' Finish initialization
	Call Initialize2(False)
	
	Dim o,q,devices: devices = Load_Devices()
	
	Dim starttime: starttime = Now
	Dim pass: pass = 1
	
	Do Until ((pass > maxRunningPasses) And DateDiff("n", starttime, Now) > maxRunningTime)
		Dim i: For i=LBound(devices) To UBound(devices)
			WScript.Echo "*** Processing [" & LPad(i+1,5) & "/" & UBound(devices)+1 & "] """ & devices(i) & """"

            Dim strCommandLine: strCommandLine = "CScript.exe //NOLOGO """
            strCommandLine = strCommandLine + WScript.ScriptFullName
            strCommandLine = strCommandLine + """ /SERVER:" + devices(i)
            If strVF <> 0 Then
                strCommandLine = strCommandLine + " /VF:" + strVF
            End If
		
			Call WSH.Run(strCommandLine, 7, False)
			
			' Try to save system resources
			If (CLng(i) mod 4) = 2 Then
				' CPU load
				Do While True
					If CPUload() > maxCPUloadHost Then
						WScript.Sleep 3000
					Else
						Exit Do
					End If
				Loop
				' Threads
				Do While True
					Set q = WMI.ExecQuery("select * from Win32_Process WHERE Name='cscript.exe' AND CommandLine LIKE '%/SERVER:%'")
					If q.Count >= maxThreadCount Then
						WScript.Sleep 1000
					Else
						Exit Do
					End If
				Loop
			End If
		Next
		
		' Wait for all threads to finish
		Do While True
			Set q = WMI.ExecQuery("select * from Win32_Process WHERE Name='cscript.exe' AND CommandLine LIKE '%/SERVER:%'")
			If q.Count > 0 Then
				WScript.Sleep 300
			Else
				Exit Do
			End If
		Loop
		
		' Check status
		Dim j, iDone, iLoad, iError, iOff, iWork, iAdm: iDone = 0: iLoad = 0: iError = 0: iOff = 0: iWork = 0: iAdm = 0
		For j=LBound(devices) To UBound(devices)
			Dim path: path = OutputPath & devices(j) & FilePrefix
			If FSO.FileExists(path & "Done") Then: iDone=iDone+1: End If
			If FSO.FileExists(path & "LOAD") Then: iLoad=iLoad+1: End If
			If FSO.FileExists(path & "Error") Then: iError=iError+1: End If
			If FSO.FileExists(path & "offline") Then: iOff=iOff+1: End If
			If FSO.FileExists(path & "noadmin") Then: iAdm=iAdm+1: End If
			If FSO.FileExists(path & "workstation") Then: iWork=iWork+1: End If
		Next
		
		WScript.Echo VbCrLf
		WScript.Echo "*** Status     [" & LPad(iDone,5)  & "/" & UBound(devices)+1 & "] " & "DONE"
		WScript.Echo "*** Status     [" & LPad(iLoad,5)  & "/" & UBound(devices)+1 & "] " & "LOAD"
		WScript.Echo "*** Status     [" & LPad(iError,5) & "/" & UBound(devices)+1 & "] " & "ERROR"
		WScript.Echo "*** Status     [" & LPad(iOff,5)   & "/" & UBound(devices)+1 & "] " & "OFFLINE"
		If iAdm > 0 Then
			WScript.Echo "*** Status     [" & LPad(iAdm,5)   & "/" & UBound(devices)+1 & "] " & "NOADMIN"
		End If
		If iWork > 0 Then
			WScript.Echo "*** Status     [" & LPad(iWork,5)   & "/" & UBound(devices)+1 & "] " & "WORKSTATIONS"
		End If
		WScript.Echo VbCrLf
		If iDone+iWork = UBound(devices)-LBound(devices)+1 Then: Exit Do: End If
		
		WScript.Sleep 20000
		pass = pass+1
	Loop
	
End Sub

'********************************************
'Purpose:
'Create a status file with computername and timestamp
Sub Touch(filename)
	On Error Resume Next
	
	Dim File: Set File = FSO.CreateTextFile(OutputPath & strComputer & FilePrefix & filename, True, True)
	If strAliasComputerName = "" Then: strAliasComputerName = strComputer: End If
	Dim fnow: fnow = Year(Now) & "-" & Right("0" & Month(Now),2) & "-" & Right("0" & Day(Now),2) & " " & FormatDateTime(Now(),vbLongTime)
	File.WriteLine CSV(Array(strAliasComputerName,fnow))
	File.Close
End Sub

'********************************************
'Purpose:
'Extend a proposed filename to a full path
Function ExtendFilename(filename)
	On Error Resume Next
	
	If Left(filename,1) = "#" Then
		ExtendFilename = OutputPath & filename
	Else
		ExtendFilename = OutputPath & strComputer & FilePrefix & filename
	End If
End Function

'********************************************
'Purpose:
'Extend a proposed filename to a full path
Function ExtendFilenameWithQuotes(filename)
	On Error Resume Next

    Dim ExtendedFileName: ExtendedFileName = ExtendFilename(filename)
    ExtendFilenameWithQuotes = """" & ExtendedFileName & """"
	
End Function

'********************************************
'Purpose:
'Create and open a text file in with unicode encoding
Function CreateFile(filename)
	On Error Resume Next
	
	FSO.DeleteFile(ExtendFilename(filename))
	Set CreateFile = FSO.CreateTextFile(ExtendFilename(filename), True, True)
End Function

'********************************************
'Purpose:
'Create or open file based on the value of bFlag - used for reentrant functions with output
Function CreateOrOpenFile(filename, bFlag)
	On Error Resume Next
	Err.clear
	
	If bFlag = False Then
		Set CreateOrOpenFile = CreateFile(filename)
	Else
		Set CreateOrOpenFile = FSO.OpenTextFile(ExtendFilename(filename), 8, True, -1)
	End If
End Function

'********************************************
'Purpose:
'Create and open a text file with UTF8 encoding
Function CreateUTF8File()
	On Error Resume Next
	
	Set CreateUTF8File = CreateObject("ADODB.Stream")
    CreateUTF8File.Type = 2 'Specify stream type - we want To save text/string data.
    CreateUTF8File.Charset = "utf-8" 'Specify charset For the source text data.
    CreateUTF8File.Open 'Prepare stream for writing
End Function

'********************************************
'Purpose:
'Write String to UTF8 file and append CRLF
Sub WriteLineToUTF8File(fileObject, strText)
	On Error Resume Next

    fileObject.WriteText strText & vbCRLF
End Sub

'********************************************
'Purpose:
'Save UTF8 text file 
Sub SaveUTF8File(fileObject, strFileName)
	On Error Resume Next
	
    fileObject.SaveToFile ExtendFilename(strFileName), 2
End Sub

'********************************************
'Purpose:
'Determine if file was saved and contains data
Function FileOK(filename)
	On Error Resume Next
	Err.clear
	
	Dim File: Set File = FSO.GetFile(ExtendFilename(filename))
	If Err = 0 Then
		If File.Size > minBytesNeeded Then
			FileOK = True
		Else
			FileOK = False
		End If
	Else
		FileOK = False
	End If
End Function

'********************************************
'Purpose:
'Does the specified file contain inventory data
Function FileIsEmpty(file)
	On Error Resume Next
	Err.clear

    FileIsEmpty = Not FileOK(file)

End Function

'********************************************
'Purpose:
'Deletes errorneous output files
Function FileOKOrDelete(filename)
	On Error Resume Next
	Err.clear
	
	If Not FileOK(filename) Then
		FSO.DeleteFile(ExtendFilename(filename))
	End If
End Function



'********************************************
'Purpose:
'Convert input array to a quoted CSV representation (tab-separated, ""-quoted)
Function CSV(arr)
	On Error Resume Next
	Err.clear
	
	Dim quote: quote = chr(34)
	Dim sep:   sep   = chr(9) 'Horizontal Tab
	Dim str
	
	CSV = ""
	
	Dim i: For i = LBound(arr) to UBound(arr)
		str = "" & arr(i)
		CSV = CSV & quote & Replace(str, quote, quote & quote) & quote & sep
	Next
End Function

'********************************************
'Purpose:
'Convert input array to a quoted CSV representation (csv-separated, ""-quoted)
Function CSV2(arr)
	On Error Resume Next
	Err.clear
	
	Dim quote: quote = chr(34)
	Dim sep:   sep   = chr(44) '","
	Dim str
	
	CSV2 = ""
	
	Dim i: For i = LBound(arr) to UBound(arr)
        If IsNumeric(arr(i)) Then
            CSV2 = CSV2 & arr(i)
        Else
            str = "" & arr(i)
		    CSV2 = CSV2 & quote & Replace(str, quote, quote & quote) & quote
        End If

        If i < UBound(arr) Then
            CSV2 = CSV2 & sep
        End If
	Next
End Function

'********************************************
'Purpose:
'Set a statusfile and write error details to WMIError
Function ErrorLog(message,number,description,source)
	On Error Resume Next
	Dim str
	
	str = CSV(Array(strComputer, message, number, source, description))
	WScript.Echo "** Error connecting to " & strComputer & ": " & description & " **"
	If number=462 Then
		Touch("offline")
	Else
		Touch("Error")
	End If
	Dim File: Set File = CreateFile("WMIError")
	File.WriteLine str
	File.Close
End Function

'********************************************
'Purpose:
'Execute CSVDE to export all objects from Active Directory. CSVDE must be installed.
Function RunCSVDE ()
	On Error Resume Next
    Err.clear

    Dim strOutputFileName1: strOutputFileName1 = ExtendFilenameWithQuotes("CSVDE_AD_Export.txt")
    Dim strOutputFileName2: strOutputFileName2 = ExtendFilename("CSVDE_AD_Export.txt")
    Dim strFullCommand: strFullCommand = "csvde -f " + strOutputFileName1
    
    ' Delete the output file if it already exists
    Call FSO.DeleteFile(strOutputFileName2, true)

    ' Create Shell Object
    Dim objShell: Set objShell = CreateObject("Wscript.Shell")

    ' Will wait for CSVDE to finish execution
    Dim retVal: retVal = objShell.Run(strFullCommand, 1, true)

    ' Check if the output file exists
    If (Not FSO.FileExists(strOutputFileName2)) Then
        ' The output file was not created. CSVDE is probably not available on this computer
        WScript.Echo "*** CSVDE export failed  ***"
        Dim File: Set File = CreateFile("CSVDE_AD_Export.txt")
	    File.WriteLine "*** CSVDE export failed  ***"
	    File.Close
    End If

End Function

'********************************************
'Purpose:
'Execute PowerShell script. PowerShell script should be in same folder as VBScript
Function RunPSScript (strScriptName, strParams)
	On Error Resume Next

	Dim strFullScriptName: strFullScriptName = """" & FullPath & "PSScripts\" & strScriptName & """"
    Dim strFullCommand: strFullCommand = "powershell.exe -ExecutionPolicy Bypass -File " & strFullScriptName & strParams
    
    'Create Shell Object
    Dim objShell: Set objShell = CreateObject("Wscript.Shell")

    'Will wait for the powershell script to finish execution
    Dim retVal: retVal = objShell.Run(strFullCommand, 1, true)

    RunPSScript = retVal

End Function

'********************************************
'Purpose:
'Execute Active Directory PowerShell script.
Function RunActiveDirectoryPSScript ()
	On Error Resume Next

    Dim strParams: strParams = ""

    strParams = strParams + " -o1 " + ExtendFilenameWithQuotes("ADDomains.csv")
    strParams = strParams + " -o2 " + ExtendFilenameWithQuotes("ADDomainTrusts.csv")
	strParams = strParams + " -o3 " + ExtendFilenameWithQuotes("ADDomainNETBIOS.csv")
	strParams = strParams + " -o4 " + ExtendFilenameWithQuotes("ADDomainControllers.csv")
	strParams = strParams + " -o5 " + ExtendFilenameWithQuotes("ADUsers.csv")
	strParams = strParams + " -o6 " + ExtendFilenameWithQuotes("ADDevices.csv")
    strParams = strParams + " -o7 " + ExtendFilenameWithQuotes("ADGroups.csv")
	strParams = strParams + " -o8 " + ExtendFilenameWithQuotes("ADExchangeServers.csv")
	strParams = strParams + " -o9 " + ExtendFilenameWithQuotes("ADActiveSyncDevices.csv")
	strParams = strParams + " -o10 " + ExtendFilenameWithQuotes("ADSettings.csv")
    strParams = strParams + " -log " + ExtendFilenameWithQuotes("ADLog.txt")
    strParams = strParams + " -verbose "

    RunPSScript "Get-ADDetails.ps1", strParams

    RunActiveDirectoryPSScript = FileOK("ADDomains.csv")

End Function

'********************************************
'Purpose:
'Execute Hyper-V PowerShell script.
Function RunHyperVPSScript (strServerName)
	On Error Resume Next

    Dim strParams: strParams = ""

    strParams = strParams + " -server " + strServerName
    strParams = strParams + " -o1 " + ExtendFilenameWithQuotes("HyperVExportBasic.csv")
    strParams = strParams + " -o2 " + ExtendFilenameWithQuotes("HyperVExportGuests.csv")
	strParams = strParams + " -o3 " + ExtendFilenameWithQuotes("HyperVExportHosts.csv")
	strParams = strParams + " -o4 " + ExtendFilenameWithQuotes("HyperVExportVMStartEvents.csv")
    strParams = strParams + " -log " + ExtendFilenameWithQuotes("HyperVLog.txt")
    strParams = strParams + " -RequiredData AllData "
    strParams = strParams + " -verbose "

    RunPSScript "Get-HyperVVMList.ps1", strParams

    RunHyperVPSScript = FileOK("HyperVExportBasic.csv")

End Function

'********************************************
'Purpose:
'Execute InstalledSoftware PowerShell script.
Function RunInstalledSoftwarePSScript (strDeviceName)
	On Error Resume Next

    RunInstalledSoftwarePSScript = FileOK("Applications.csv")
    If RunInstalledSoftwarePSScript = True Then
        ' We've already got an Applications export from this device.
        Exit Function
    End If

    Dim strParams: strParams = ""

    strParams = strParams + " -target " + strDeviceName
    strParams = strParams + " -o1 " + ExtendFilenameWithQuotes("Applications.csv")
    strParams = strParams + " -log " + ExtendFilenameWithQuotes("ApplicationsLog.txt")

    RunPSScript "Get-InstalledSoftware.ps1", strParams

    RunInstalledSoftwarePSScript = FileOK("Applications.csv")

End Function

'********************************************
'Purpose:
'Execute SharePoint PowerShell script.
Function RunSharePointPSScript (strDeviceName, bHeadless)
	On Error Resume Next

    Dim strParams: strParams = ""

    If Not (IsStringEmptyOrNull(strDeviceName)) Then
        strParams = strParams + " -server " + strDeviceName
    End If

    strParams = strParams + " -o1 " + ExtendFilenameWithQuotes("SharePointSites.csv")
    strParams = strParams + " -o2 " + ExtendFilenameWithQuotes("SharePointUserGroups.csv")
    strParams = strParams + " -o3 " + ExtendFilenameWithQuotes("SharePointUserCALs.csv")
    strParams = strParams + " -i1 " + ExtendFilenameWithQuotes("ADGroups.csv")
    strParams = strParams + " -i2 " + ExtendFilenameWithQuotes("ADUsers.csv")
    strParams = strParams + " -log " + ExtendFilenameWithQuotes("SharePointLog.txt")

    If bHeadless = True Then
        strParams = strParams + " -headless"
    Else
        WScript.Echo ""
        WScript.Echo " Running SharePoint PowerShell Script"
        WScript.Echo " This script requires credentials and PSExec to execute on remote SharePoint"
        WScript.Echo " servers. It should be run once per SharePoint Farm"
        WScript.Echo " Output logged to " & ExtendFilename("SharePointLog.txt")
        WScript.Echo ""
    End If

    If strUser <> "<notset>" And strPassword <> "<notset>" Then
        strParams = strParams + " -username " + strUser
        strParams = strParams + " -password " + strPassword
    End If

    RunPSScript "Get-SharePointLicenseDetails.ps1", strParams

    RunSharePointPSScript = FileOK("SharePointSites.csv")

End Function

'********************************************
'Purpose:
'Execute Exchange Server PowerShell script.
Function RunExchangePSScript ()
	On Error Resume Next

    Dim strParams: strParams = ""

    WScript.Echo ""
    WScript.Echo " Running Exchange Server PowerShell Script"
    WScript.Echo " This script should be run once per Exchange Server Site"
    WScript.Echo " Output logged to " & ExtendFilename("ExchangeQueryLog.txt")
    WScript.Echo ""

    strParams = strParams + " -o1 " + ExtendFilenameWithQuotes("ExchangeServerDetails.csv")
    strParams = strParams + " -o2 " + ExtendFilenameWithQuotes("ExchangeMailBoxes.csv")
    strParams = strParams + " -o3 " + ExtendFilenameWithQuotes("ExchangeDevices.csv")
    strParams = strParams + " -o4 " + ExtendFilenameWithQuotes("ExchangeCALs.csv")
    strParams = strParams + " -o5 " + ExtendFilenameWithQuotes("ExchangeCALDetails.csv")
    strParams = strParams + " -log " + ExtendFilenameWithQuotes("ExchangeQueryLog.txt")

    Dim strExchangeServer: strExchangeServer = args("/EXCHANGE:")
    If Not (IsStringEmptyOrNull(strExchangeServer)) Then
        strParams = strParams + " -server " + strExchangeServer
    End If

    If strUser <> "<notset>" Then
        strParams = strParams + " -username " + GetFullUserName
        If strPassword <> "<notset>" Then
            strParams = strParams + " -password " + strPassword
        End If
    End If

    strParams = strParams + " -RequiredData AllData"
    strParams = strParams + " -ConnectionMethod Both"  
    strParams = strParams + " -Verbose"
    
    RunPSScript "Get-ExchangeDetails.ps1", strParams

    RunExchangePSScript = FileOK("ExchangeServerDetails.csv")

End Function

'********************************************
'Purpose:
'Execute Lync Server PowerShell script.
Function RunLyncPSScript ()
	On Error Resume Next

    Dim strParams: strParams = ""

    WScript.Echo ""
    WScript.Echo " Running Lync Server PowerShell Script"
    WScript.Echo " Output logged to " & ExtendFilename("LyncQueryLog.txt")
    WScript.Echo ""

    Dim strLyncServer: strLyncServer = args("/LYNC:")

    strParams = strParams + " -o1 " + ExtendFilenameWithQuotes("LyncUsers.csv")
    strParams = strParams + " -log " + ExtendFilenameWithQuotes("LyncQueryLog.txt")

    If Not (IsStringEmptyOrNull(strLyncServer)) Then
        strParams = strParams + " -server " + strLyncServer
    End If

    If strUser <> "<notset>" Then
        strParams = strParams + " -username " + GetFullUserName
        If strPassword <> "<notset>" Then
            strParams = strParams + " -password " + strPassword
        End If
    End If

    strParams = strParams + " -Verbose"
    
    RunPSScript "Get-LyncUsers.ps1", strParams

    RunLyncPSScript = FileOK("LyncUsers.csv")

End Function

'********************************************
'Purpose:
'Execute VMware VM query PowerShell script.
Function RunVMwareQueryPSScript ()
	On Error Resume Next

    Dim strParams: strParams = ""

    WScript.Echo ""
    WScript.Echo " Running VMware PowerShell Script"
    WScript.Echo " Output logged to " & ExtendFilename("VMQueryLog.txt")
    WScript.Echo ""

    Dim strVMwareServer: strVMwareServer = args("/VMWARE:")

    strParams = strParams + " -o1 " + ExtendFilenameWithQuotes("VMwareVMs.csv")
    strParams = strParams + " -o2 " + ExtendFilenameWithQuotes("VMwareHosts.csv")
    strParams = strParams + " -o3 " + ExtendFilenameWithQuotes("VMwareClusters.csv")
    strParams = strParams + " -o4 " + ExtendFilenameWithQuotes("VMwareVMDRSRules.csv")
    strParams = strParams + " -o5 " + ExtendFilenameWithQuotes("VMwareVMDRSGroups.csv")
    strParams = strParams + " -o6 " + ExtendFilenameWithQuotes("VMwareVMotionData.csv")
    strParams = strParams + " -log " + ExtendFilenameWithQuotes("VMQueryLog.txt")
    strParams = strParams + " -VMotionHistoryDaysValue 90"

    If Not (IsStringEmptyOrNull(strVMwareServer)) Then
        strParams = strParams + " -server " + strVMwareServer
    End If

    If strUser <> "<notset>" Then
        strParams = strParams + " -username " + GetFullUserName
        If strPassword <> "<notset>" Then
            strParams = strParams + " -password " + strPassword
        End If
    End If

    strParams = strParams + " -Verbose"
    strParams = strParams + " -RequiredData AllData"
    
    RunPSScript "Get-VMwareVMList.ps1", strParams

    RunVMwareQueryPSScript = FileOK("VMwareData.csv")

End Function


'********************************************
'Purpose:
'Execute Office365 License Entitlement query PowerShell script.
Function RunOffice365QueryPSScript ()
	On Error Resume Next

    Dim strParams: strParams = ""

    strParams = strParams + " -o1 " + ExtendFilenameWithQuotes("Office365Licenses.csv")
    strParams = strParams + " -o2 " + ExtendFilenameWithQuotes("Office365LicenseAssignments.csv")
    strParams = strParams + " -log " + ExtendFilenameWithQuotes("Office365QueryLog.txt")

    If strUser <> "<notset>" Then
        strParams = strParams + " -username " + GetFullUserName
        If strPassword <> "<notset>" Then
            strParams = strParams + " -password " + strPassword
        End If
    End If
    
    RunPSScript "Get-Office365LicenseDetails.ps1", strParams

    RunOffice365QueryPSScript = FileOK("Office365UserData.csv")

End Function

'********************************************
'Purpose:
'Execute Azure VM query PowerShell script.
Function RunAzureVMQueryPSScript ()
	On Error Resume Next

    Dim strParams: strParams = ""

    strParams = strParams + " -o1 " + ExtendFilenameWithQuotes("AzureClassicVMList.csv")
    strParams = strParams + " -o2 " + ExtendFilenameWithQuotes("AzureRMVMList.csv")
    strParams = strParams + " -log " + ExtendFilenameWithQuotes("AzureQueryLog.txt")

    If strUser <> "<notset>" Then
        strParams = strParams + " -username " + GetFullUserName
        If strPassword <> "<notset>" Then
            strParams = strParams + " -password " + strPassword
        End If
    End If
    
    RunPSScript "Get-AzureVMList.ps1", strParams

    RunAzureVMQueryPSScript = FileOK("AzureVMList.csv")

End Function

'********************************************
'Purpose:
'Create global FileSystemObject instance
Sub ConnectFSO
	On Error Resume Next
	Err.clear
	
	Set FSO = CreateObject("Scripting.FileSystemObject")
	If Err<>0 Then
        Call ErrorLog("FSO",Err.number,Err.description,Err.source)
        WScript.Echo "** Error initialising Scripting.FileSystemObject component. Script exiting. **"
	    WScript.Quit
    End If
End Sub

'********************************************
'Purpose:
'Create global WScript.Shell instance
Sub ConnectWSH
	On Error Resume Next
	Err.clear
	
	Set WSH = CreateObject("WScript.Shell")
	If Err<>0 Then
        Call ErrorLog("WSH",Err.number,Err.description,Err.source)
        WScript.Echo "** Error initialising WScript.Shell component. Script exiting. **"
	    WScript.Quit
    End If
End Sub

'********************************************
'Purpose:
'Create global LDAP instance and retrieve DefaultNamingContext
Sub ConnectDSE
	On Error Resume Next
	Err.clear
	
	Set DSE = GetObject("LDAP://RootDSE")
	
	' Default Naming Context
	DNC = DSE.Get("DefaultNamingContext")
	If Trim(DNC) = "" Then
		WScript.Echo "No domain found"
		bADDomainFound = False
	Else
		bADDomainFound = True
	End If
End Sub
'********************************************
'Purpose:
'Create global Windows Management Instrumentation instance for cimv2 and registry
Function ConnectWMI
	On Error Resume Next
	Err.clear

    ConnectWMI = FALSE
	
	If strUser <> "<notset>" And strPassword <> "<notset>" And strDomain <> "<notset>" Then
		' Initialise SWbemLocator
        Dim SWL: Set SWL = CreateObject("WbemScripting.SWbemLocator")
		If Err<>0 Then
            Call ErrorLog("SWbemLocator",Err.number,Err.description,Err.source)
            WScript.Echo "** Error initialising SWbemLocator component **"
	        Exit Function
        End If
        SWL.Security_.AuthenticationLevel = wbemAuthenticationLevelPkt
		If Err<>0 Then
            Call ErrorLog("SWbemLocator",Err.number,Err.description,Err.source)
            WScript.Echo "** Error configuring SWbemLocator component **"
	        Exit Function
        End If
		
        ' Explicit user credentials
		Set WMI = SWL.ConnectServer(strComputer,"root\cimv2",strUser,strPassword,"MS_409","NTLMDomain:" & strDomain, 128)
		If Err<>0 Then
            Call ErrorLog("WMI",Err.number,Err.description,Err.source)
            WScript.Echo "** Error connecting WMI (cimv2) **"
	        Exit Function
        End If
		Call Touch("online"): Err.clear

		Dim DEF: Set DEF = SWL.ConnectServer(strComputer,"root\default",strUser,strPassword,"MS_409","NTLMDomain:" & strDomain, 128)
		If Err<>0 Then
            Call ErrorLog("REG",Err.number,Err.description,Err.source)
            WScript.Echo "** Error connecting WMI (default) **"
	        Exit Function
        End If
		
        Set REG = DEF.Get("StdRegProv")
		If Err<>0 Then
            Call ErrorLog("REG",Err.number,Err.description,Err.source)
            WScript.Echo "** Error connecting WMI (StdRegProv) **"
	        Exit Function
        End If
	Else
        ' Use credentials of current user
		Set WMI = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & strComputer & "\root\cimv2")
		If Err<>0 Then
            Call ErrorLog("WMI",Err.number,Err.description,Err.source)
            WScript.Echo "** Error connecting WMI (cimv2) **"
	        Exit Function
        End If
		Call Touch("online"): Err.clear
		
        Set REG = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & strComputer & "\root\default:StdRegProv")
		If Err <> 0 Then
			Err.clear
			Set REG = GetObject("winmgmts:{impersonationLevel=impersonate,authenticationLevel=pktPrivacy}!\\" & strComputer & "\root\default:StdRegProv")
			If Err<>0 Then
                Call ErrorLog("REG",Err.number,Err.description,Err.source)
                WScript.Echo "** Error connecting WMI (StdRegProv) **"
	            Exit Function
            End If
		End If
	End If

    ConnectWMI = True
End Function
'********************************************
'Purpose:
'Create global WMI instance for microsoftdns
Sub ConnectWMIDNS
	On Error Resume Next
	Err.clear
	
	' Explicit user credentials
	If strUser <> "<notset>" And strPassword <> "<notset>" And strDomain <> "<notset>" Then
		Dim SWL: Set SWL = CreateObject("WbemScripting.SWbemLocator")
		SWL.Security_.AuthenticationLevel = wbemAuthenticationLevelPkt

		Set WMIDNS = SWL.ConnectServer(strComputer,"root\microsoftdns",strUser,strPassword,"MS_409","ntlmdomain:" & strDomain)
	' Use credentials of current user
	Else
		Set WMIDNS = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & strComputer & "\root\microsoftdns")
	End If
End Sub
'********************************************
'Purpose:
'Create global WMI instance for MS SQL Server
Sub ConnectWMISQL
	On Error Resume Next
	Err.clear
	
	' Explicit user credentials
	If strUser <> "<notset>" And strPassword <> "<notset>" And strDomain <> "<notset>" Then
		Dim SWL: Set SWL = CreateObject("WbemScripting.SWbemLocator")
		SWL.Security_.AuthenticationLevel = wbemAuthenticationLevelPkt

        ' 2005
		Set WMISQL(1) = SWL.ConnectServer(strComputer,"root\Microsoft\SqlServer\ComputerManagement",strUser,strPassword,"MS_409","ntlmdomain:" & strDomain)
		If Err <> 0 Then
			Err.clear: WMISQL(1) = Null
		End If

        ' 2008
		Set WMISQL(2) = SWL.ConnectServer(strComputer,"root\Microsoft\SqlServer\ComputerManagement10",strUser,strPassword,"MS_409","ntlmdomain:" & strDomain)
		If Err <> 0 Then
			Err.clear: WMISQL(2) = Null
		End If

        ' 2012
		Set WMISQL(3) = SWL.ConnectServer(strComputer,"root\Microsoft\SqlServer\ComputerManagement11",strUser,strPassword,"MS_409","ntlmdomain:" & strDomain)
		If Err <> 0 Then
			Err.clear: WMISQL(3) = Null
		End If

        ' 2014
		Set WMISQL(4) = SWL.ConnectServer(strComputer,"root\Microsoft\SqlServer\ComputerManagement12",strUser,strPassword,"MS_409","ntlmdomain:" & strDomain)
		If Err <> 0 Then
			Err.clear: WMISQL(4) = Null
		End If

        ' 2016
		Set WMISQL(5) = SWL.ConnectServer(strComputer,"root\Microsoft\SqlServer\ComputerManagement13",strUser,strPassword,"MS_409","ntlmdomain:" & strDomain)
		If Err <> 0 Then
			Err.clear: WMISQL(5) = Null
		End If
	Else
        ' Use credentials of current user
		' 2005
        Set WMISQL(1) = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & strComputer & "\root\Microsoft\SqlServer\ComputerManagement")
		If Err <> 0 Then
			Err.clear: WMISQL(1) = Null
		End If

        ' 2008
        Set WMISQL(2) = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & strComputer & "\root\Microsoft\SqlServer\ComputerManagement10")
		If Err <> 0 Then
			Err.clear: WMISQL(2) = Null
		End If

        ' 2012
        Set WMISQL(3) = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & strComputer & "\root\Microsoft\SqlServer\ComputerManagement11")
		If Err <> 0 Then
			Err.clear: WMISQL(3) = Null
		End If

        ' 2014
        Set WMISQL(4) = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & strComputer & "\root\Microsoft\SqlServer\ComputerManagement12")
		If Err <> 0 Then
			Err.clear: WMISQL(4) = Null
		End If

        ' 2016
        Set WMISQL(5) = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & strComputer & "\root\Microsoft\SqlServer\ComputerManagement13")
		If Err <> 0 Then
			Err.clear: WMISQL(5) = Null
		End If
	End If
End Sub
'********************************************
'Purpose:
'Create global WMI instance for MS Cluster service
Sub ConnectWMICLUS
	On Error Resume Next
	Err.clear
	
	Dim SWL: Set SWL = CreateObject("WbemScripting.SWbemLocator")
	SWL.Security_.AuthenticationLevel = wbemAuthenticationLevelPkt
	
	' Explicit user credentials
	If strUser <> "<notset>" And strPassword <> "<notset>" And strDomain <> "<notset>" Then
		Set WMICLUS = SWL.ConnectServer(strComputer,"root\MSCluster",strUser,strPassword,"MS_409","ntlmdomain:" & strDomain)
	' Use credentials of current user
	Else
		Set WMICLUS = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & strComputer & "\root\MSCluster")
	End If
	
	Err.clear
	Dim q: Set q = WMICLUS.ExecQuery("Select * from MSCluster_Cluster")
	' Try to encrypt WMI if first method fails (MS Server 2008 issue)
	If Err <> 0 Then
		Err.clear
		' Explicit user credentials
		If strUser <> "<notset>" And strPassword <> "<notset>" And strDomain <> "<notset>" Then
			SWL.Security_.Privileges.AddAsString "SeSecurityPrivilege"
			Set WMICLUS = SWL.ConnectServer(strComputer,"root\MSCluster",strUser,strPassword,"MS_409","ntlmdomain:" & strDomain)
		' Use credentials of current user
		Else
			Set WMICLUS = GetObject("winmgmts:{impersonationLevel=impersonate,authenticationLevel=pktPrivacy}!\\" & strComputer & "\root\MSCluster")
		End If
	End If
	Err.clear
End Sub
'********************************************
'Purpose:
'Create global WMI instance for HyperV
Sub ConnectWMIVIRT
	On Error Resume Next
	Err.clear
	
	' Explicit user credentials
	If strUser <> "<notset>" And strPassword <> "<notset>" And strDomain <> "<notset>" Then
		Dim SWL: Set SWL = CreateObject("WbemScripting.SWbemLocator")
		SWL.Security_.AuthenticationLevel = wbemAuthenticationLevelPkt

		Set WMIVIRT = SWL.ConnectServer(strComputer,"root\virtualization",strUser,strPassword,"MS_409","ntlmdomain:" & strDomain)
	' Use credentials of current user
	Else
		Set WMIVIRT = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & strComputer & "\root\virtualization")
	End If
End Sub
'********************************************
'Purpose:
'Execute a WMI query against provided instance and return first result
Function Query1(WMI, query)
	On Error Resume Next
	Err.clear
	
	Set Query1 = Nothing

	If Not IsNull(WMI) Then
		Dim q,r: Set q = WMI.ExecQuery(query)
		If Err = 0 Then
			If q.Count > 0 Then: For Each r In q: Set Query1 = r: Exit Function: Next: End If
		End If
	End If
End Function
'********************************************
'Purpose:
'Return username and domain name in one string
Function GetFullUserName
	On Error Resume Next
	Err.clear

	If (IsNull(strUser) Or IsEmpty(strUser) Or strUser = "<notset>") Then
        ' No user name specified
        GetFullUserName = ""
        Exit Function
    End If

    If (InStr(strUser, "@") > 0) Or (InStr(strUser, "\") > 0) Then
        ' Domain name is already in user name
        GetFullUserName = strUser
        Exit Function
    End If

    If Not (IsNull(strDomain) Or IsEmpty(strDomain) Or strDomain = "<notset>") Then
        ' A domain name has been specified
        If (InStr(strDomain, ".") > 0) Then
            ' Domain is in FQDN format
            GetFullUserName = strUser + "@" + strDomain
        Else
            ' Domain is in NetBIOS format
            GetFullUserName = strDomain + "\" + strUser
        End If

        Exit Function
    End If

    ' Only a username was specified
    GetFullUserName = strUser

End Function

'********************************************
'********************************************
'********************************************
'Purpose:
'Determine the correct local name of the computer
Sub Computer_LocalName
	On Error Resume Next
	Err.clear
	
	Dim o: Set o = WScript.CreateObject("WScript.Network")
	If Err = 0 Then: strLocalComputerName = UCase(Trim(o.ComputerName)): Else: strLocalComputerName = "" End If
End Sub

'********************************************
'Purpose:
'Determine the domain role
Sub Computer_DomainRole
	On Error Resume Next
	Err.clear
	
	With Query1(WMI, "Select DomainRole from Win32_ComputerSystem")
		Select Case .DomainRole
			Case 0: strDomainRole = "Standalone Workstation"
			Case 1: strDomainRole = "Member Workstation"
			Case 2: strDomainRole = "Standalone Server"
			Case 3: strDomainRole = "Member Server"
			Case 4: strDomainRole = "Backup Domain Controller"
			Case 5: strDomainRole = "Primary Domain Controller"
		End Select
	End With
End Sub

'********************************************
'Purpose:
'Gather and write general system related information, e.g OS Version, Memory, Domain name, BIOS Manufacturer (VMware detection)
Function System
	On Error Resume Next
	Err.clear
	
	Dim lines: lines = ""
	
	WScript.Echo "------- System information from " & strComputer & " -------"
	
	With Query1(WMI, "Select * from Win32_OperatingSystem")
		strRealComputerName = UCase(.CSName)
		strComputer = UCase(strComputer)
		
		' Backwards compatibility
		If strRealComputerName <> strComputer Then: strAliasComputerName = LCase(strComputer): End If
		
		If Instr(UCase(.Caption),"SERVER") > 0 Then
			' Server
			WScript.Echo "------- " & strComputer & " is a server"
		Else
			' Workstation
			WScript.Echo "------- " & strComputer & " is a workstation"
			bIsWorkstation = True
			If bOnlyCheckServers Then
				Call Touch("workstation")
				WScript.Echo "*** SKIPPING " & strComputer & " because it is a workstation"
				WScript.Quit
			End If
		End If
	    
		' Local user output
		lines = lines & CSV(Array(strAliasComputerName, .CSName, .Caption, .OtherTypeDescription))
		
		' 
		With Query1(WMI, "Select * from Win32_ComputerSystem")
			strComputerDomain = "" & .Domain
			lines = lines & CSV(Array(strComputerDomain, FormatNumber(.TotalPhysicalMemory/1024/1024,0) & "MB", .Model, .Manufacturer))
		End With
		
		With Query1(WMI, "Select * from Win32_BIOS")
			lines = lines & CSV(Array(.Manufacturer))
		End With
		
		lines = lines & CSV(Array(ScriptName))
	End With
	
    ' Creating CSV file for System info
	Dim File: Set File = CreateFile("System.txt")
    File.WriteLine CSV(Array("ComputerName", "Name", "Caption", "TypeDescription", "ComputerDomain",_
                             "TotalPhysicalMemory", "Model", "Manufacturer", "BiosManufacturer", "ScriptName"))

	File.WriteLine lines

	File.Close
	System = FileOK("System.txt")

End Function
'********************************************
'********************************************
'********************************************
'********************************************
'Purpose:
'Gather and write information about running processes and call hook function for extra processing
Function Processes
	On Error Resume Next
	Err.clear
	
	WScript.Echo "------- Process Information from "& strComputer & " -------"	
	
    ' Creating CSV file for Processes info
	Dim File: Set File = CreateFile("Processes.txt")
    File.WriteLine CSV(Array("ComputerName", "Name", "ExecutablePath"))
		
	Dim o,q: Set q = WMI.ExecQuery("Select * from Win32_Process")
	For Each o In q
	With o
		Call Processes_Hook(.Name)
		
		File.WriteLine CSV(Array(strAliasComputerName, .Name, .ExecutablePath))
	End With
	Next
	
	File.Close
	Processes = FileOK("Processes.txt")

End Function
'********************************************
'Purpose:
'Allows extra processing of running processes based on their name
Sub Processes_Hook(name)
	On Error Resume Next
	Err.clear
End Sub
'********************************************
'Purpose:
'Gather and write information about running services and call hook function for extra processing
Function Services
	On Error Resume Next
	Err.clear
	
	Dim bInclude
	
	WScript.Echo "------- Service Information from "& strComputer & " -------"
	
    ' Creating CSV file for Services info
	Dim File: Set File = CreateFile("Services.txt")
    File.WriteLine CSV(Array("ComputerName", "DisplayName", "Name", "State", "StartMode", "path"))
	
	Dim o,q: Set q = WMI.ExecQuery("Select * from Win32_Service")
	For Each o In q
	With o
		Dim strDisplay: strDisplay = ""&.DisplayName
		Call Services_Hook(""&.Name, strDisplay, ""&.PathName, ""&.State)
		
		Dim path: path = Replace(""&.PathName, chr(34), "")
		If Not (InStr(""&.PathName, path) > 0) Then: path = ""&.PathName: End If
		
		bInclude = True

		If bInclude Then
			File.WriteLine CSV(Array(strAliasComputerName, strDisplay, .Name, .State, .StartMode, path))
		End If
	End With
	Next
	
	File.Close
	Services = FileOK("Services.txt")

End Function
'********************************************
'Purpose:
'Gather and write information from application event log on installations/removals
Function MSIInstaller
	On Error Resume Next
	Err.clear
	
	Dim bInclude
	
	WScript.Echo "------- MSIInstaller information from "& strComputer & " -------"
	
    ' Creating CSV file for MSI Installer Events info
	Dim File: Set File = CreateFile("EventLog.txt")
    File.WriteLine CSV(Array("ComputerName", "EventCode", "Message", "TimeWritten"))
	
	File.WriteLine CSV(Array(strAliasComputerName, "START", "START", "19000101000000.000000+000"))
	
	' MSIInstaller events 
	' ID 1033 = Installed software
	' ID 1034 = Removed software
	' ID 1035 = Reconfigured software
	' ID 1036 = Installed update
	' ID 11707 = Installed software (win2003)
	' ID 11724 = Removed software (win2003)
	Dim o,q: Set q = WMI.ExecQuery("Select * from Win32_NTLogEvent Where LogFile='Application' And SourceName Like 'msiinstaller' AND " & _
								   "((EventCode>=1033 AND EventCode<=1036) OR EventCode=11707 OR EventCode=11724)")
	For Each o In q
	With o
		' Remove newlines/pagebreaks and change ; -> ,
		Dim strMessage: strMessage = Replace(Replace(Replace("" & .Message, ";", ","), chr(13), ""), chr(10), "")
		
		bInclude = True

        If strVF <> 0 Then
            If (Not (InStr(UCase(strMessage), UCase(strVF)) > 0)) Then
                ' The current event does not match the filter
                bInclude = False
            End If
        End If

		If bInclude Then
			File.WriteLine CSV(Array(strAliasComputerName, .EventCode, strMessage, .TimeWritten))
		End If
	End With
	Next
	
	File.WriteLine CSV(Array(strAliasComputerName, "END", "END", "19000101000000.000000+000"))

	File.Close
	MSIInstaller = FileOK("EventLog.txt")

End Function

'********************************************
'Purpose:
'Gather and write information about active network configurations, i.e IP and MAC addresses
Function Computer_IP
	On Error Resume Next
	Err.clear
	
	WScript.Echo "------- IP address information from " & strComputer & " -------"
	
    ' Creating CSV file for IP and MAC addresses info
	Dim File: Set File = CreateFile("IP.txt")
    File.WriteLine CSV(Array("ComputerName", "IPAddressCount", "IPAddress", "MACAddress"))	
	
	Dim o,q: Set q = WMI.ExecQuery("Select * from Win32_NetworkAdapterConfiguration Where IPEnabled=TRUE")
	For Each o In q
	With o
		If Not IsNull(.IPAddress) Then
			Dim i, c: c = 0 
			For i=LBound(.IPAddress) To UBound(.IPAddress)
				File.WriteLine CSV(Array(strAliasComputerName, c, .IPAddress(i), .MACAddress(i)))
				c = c + 1
			Next
		End If
	End With
	Next
	
	File.Close
	Computer_IP = FileOK("IP.txt")

End Function

'********************************************
'Purpose:
'Gather and write information about the installed CPUs
Function CPU
	On Error Resume Next
	Err.clear
	
    Dim strProcessorName: strProcessorName = ""
    Dim strStatusInfo: strStatusInfo = ""
    Dim strProcessorMaxSpeed: strProcessorMaxSpeed = ""
    Dim nProcessorCount: nProcessorCount = 0
    Dim nCoresPerProcessorCount: nCoresPerProcessorCount = 0
    Dim nLogicalProcessorsPerProcessorCount: nLogicalProcessorsPerProcessorCount = 0
    Dim nProcessorCountEnabled: nProcessorCountEnabled = 0
    Dim nProcessorCountDisabled: nProcessorCountDisabled = 0
    Dim nProcessorCountOther: nProcessorCountOther = 0
    Dim strStatus: strStatus = "Success"

	WScript.Echo "------- Processor information from " & strComputer & " -------"

    Dim o,q: Set q = WMI.ExecQuery("Select * from Win32_Processor")
	For Each o In q
	With o
		' Compatibility fix for old OS versions
		Err.clear
        strProcessorName                    = .Name
        strProcessorMaxSpeed                = .MaxClockSpeed:             If Err <> 0 Then: strProcessorMaxSpeed = "Unknown": strStatus = "Warning: Old WMI": Err.Clear: End If
        nCoresPerProcessorCount             = .NumberOfCores:             If Err <> 0 Then: nCoresPerProcessorCount = 1: strStatus = "Warning: Old WMI": Err.Clear: End If
		nLogicalProcessorsPerProcessorCount = .NumberOfLogicalProcessors: If Err <> 0 Then: nLogicalProcessorsPerProcessorCount = 1: strStatus = "Warning: Old WMI": Err.Clear: End If
		strStatusInfo = .StatusInfo:                                      If Err <> 0 Then: strStatusInfo = -1: strStatus = "Warning: Old WMI": Err.Clear: End If

		Select Case strStatusInfo
			'Case 1: SI = "other"
			'Case 2: SI = "unknown"
			Case 3: nProcessorCountEnabled = nProcessorCountEnabled + 1
			Case 4: nProcessorCountDisabled = nProcessorCountDisabled + 1
			'Case 5: SI = "Not Applicaple"
            Case Else: nProcessorCountOther = nProcessorCountOther + 1
		End Select
		
		nProcessorCount = nProcessorCount + 1
	End With
	Next
    
    ' Creating CSV file for CPU info	
    Dim File: Set File = CreateFile("CPU.txt")
    File.WriteLine CSV(Array(_
        "Device","ProcessorName","MaxSpeed","ProcessorCount", "CoreCount","LogicalProcessorCount",_
        "ProcessorCountStatusEnabled","ProcessorCountStatusDisabled","ProcessorCountStatusOther","Status"))
    File.WriteLine CSV(Array(strAliasComputerName, strProcessorName, strProcessorMaxSpeed, nProcessorCount,_ 
        nCoresPerProcessorCount * nProcessorCount, nLogicalProcessorsPerProcessorCount * nProcessorCount,_
         nProcessorCountEnabled, nProcessorCountDisabled, nProcessorCountOther, strStatus))
	
    File.Close
	CPU = FileOK("CPU.txt")

End Function
'********************************************
'Purpose:
'Gather and write information about the Hard Disk space
Function DiskSpace
	On Error Resume Next
	Err.clear
	
    Dim description: description = ""
    Dim deviceID: deviceID = ""
    Dim filesystem: filesystem = ""
    Dim freespace: freespace = 0
    Dim size : size = 0
    Dim strStatus: strStatus = "Success"
    Dim strStatusInfo: strStatusInfo = ""

	WScript.Echo "------- Disk Space information from " & strComputer & " -------"

    ' Creating CSV file for DiskSpace info	
    Dim File: Set File = CreateFile("DiskSpace.txt")
    File.WriteLine CSV(Array("Device","Description","DeviceID","FileSystem","FreeSpaceGB","SizeGB","Status"))
	
	Dim o,q: Set q = WMI.ExecQuery("Select description, deviceID, filesystem, freespace, size, StatusInfo from win32_logicaldisk")
	For Each o In q
	With o

		Err.clear
        description                    		= .description
        deviceID                			= .deviceID:             	      If Err <> 0 Then: deviceID = "Unknown": Err.Clear: End If
        filesystem             				= .filesystem:                    If Err <> 0 Then: filesystem = "Unknown": Err.Clear: End If
		freespace 							= Round(.freespace/1073741824,1): If Err <> 0 Then: freespace = -1: Err.Clear: End If
		size								= Round(.size/1073741824,1): 	  If Err <> 0 Then: size = -1: Err.Clear: End If
		strStatusInfo 						= .StatusInfo:                    If Err <> 0 Then: strStatusInfo = -1: strStatus = "Warning: Error occurred": Err.Clear: End If
		
	End With
	
		File.WriteLine CSV(Array(strAliasComputerName, description, deviceID, filesystem, freespace, size, strStatus))
		
	Next
    
	
    File.Close
	DiskSpace = FileOK("DiskSpace.txt")

End Function
'********************************************
'Purpose:
'Gather and write information about installed software - delegates processing to "software_helper" 
Function Software
	On Error Resume Next
	Err.clear
	
	WScript.Echo "------- Installed Application from " & strComputer & " -------"
		
    ' Creating CSV file for Installed Software info
	Dim File: Set File = CreateFile("Applications.txt")
    File.WriteLine CSV(Array("ComputerName", "DisplayName", "InstallDate", "VersionMajor", _
                             "DisplayVersion", "Publisher"))
	
	Dim paths: paths = Array("SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\", _
							 "SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\")
	Dim path: For Each path In paths		
		Dim subKey, subKeys
		Call REG.EnumKey(HKLM, path, subKeys)
		If IsArray(subKeys) Then
			For Each subKey In subKeys
				Call Software_Helper(HKLM, path & subKey, File)
			Next
		End If

		Dim subUser, subUsers
		Call REG.EnumKey(HKU, "", subUsers)
		If IsArray(subUsers) Then
			For Each subUser In subUsers
				Call REG.EnumKey(HKU, subUser & "\" & path, subKeys)
				If IsArray(subKeys) Then
					For Each subKey In subKeys
						Call Software_Helper(HKU, subUser & "\" & path & subKey, File)
					Next
				End If
			Next
		End If
	Next
	
	Call Software_IE(HKLM, "SOFTWARE\Microsoft\Internet Explorer", File) ' get real IE version and add it as additional line

	File.Close
	Software = FileOK("Applications.txt")

End Function

'Purpose:
'Gather and write information about installed software and calls hook function for extra processing - called from "software"
Sub Software_Helper(hive, path, File)
	On Error Resume Next
	Err.clear
	
	Dim vals(6)

	If REG.GetStringValue(hive, path, "DisplayName", vals(0)) <> 0 Then
		Call REG.GetStringValue(hive, path, "QuietDisplayName", vals(0))
	End If
	If Trim(vals(0)) <> "" Then
		Call Software_Hook(vals(0), hive, path, File)
		
		Call REG.GetStringValue (hive, path, "InstallDate", vals(1))
		Call REG.GetDWORDValue  (hive, path, "VersionMajor", vals(2))
		Call REG.GetDWORDValue  (hive, path, "VersionMinor", vals(3))
		Call REG.GetStringValue (hive, path, "DisplayVersion", vals(4))
		Call REG.GetStringValue (hive, path, "Publisher", vals(5))
		
		' Version
		vals(2) = Trim(vals(2)) & "." & Trim(vals(3))
		If vals(2) = "." Then: vals(2) = "": End If
		
        If strVF <> 0 Then
            If (IsNull(vals(5))) Or (IsEmpty(vals(5))) Or (Not (InStr(UCase(vals(5)), UCase(strVF)) > 0)) Then
                ' The current product record does not match the filter
                Exit Sub
            End If
        End If
		
		If bInclude Then
			File.WriteLine CSV(Array(strAliasComputerName, vals(0), vals(1), vals(2), vals(4), vals(5)))
		End If
	End If
End Sub

'Purpose:
'Gather and write information about installed Internet Explorer real version installed
Sub Software_IE(hive, path, File)
	On Error Resume Next
	Err.clear
		
	Dim vals(6)
	
	' Newer IE versions use the value svcVersion, older versions use Version, 
	' so we need to check svcVersion first and fall back to Version if svcVersion is not set. 
	If REG.GetStringValue(hive, path, "svcVersion", vals(4)) <> 0 Then
		Call REG.GetStringValue(hive, path, "Version", vals(4)) ' "DisplayVersion"
	End If
	
	If Trim(vals(4)) <> "" Then
		
		vals(0) = "Internet Explorer " + vals(4)
		vals(1) = "" ' "InstallDate"
		vals(2) = "" ' "VersionMajor"
		vals(3) = "" ' "VersionMinor"
		vals(5) = "Microsoft Corporation" ' "Publisher"
				
        If strVF <> 0 Then
            If (IsNull(vals(5))) Or (IsEmpty(vals(5))) Or (Not (InStr(UCase(vals(5)), UCase(strVF)) > 0)) Then
                ' The current product record does not match the filter
                Exit Sub
            End If
        End If
		
		If bInclude Then
			File.WriteLine CSV(Array(strAliasComputerName, vals(0), vals(1), vals(2), vals(4), vals(5)))
		End If
		
	End If

End Sub

'********************************************
'Purpose:
'Collect Software License Info from Software Licensing service via WMI
Function SoftwareLicenseInfo
	On Error Resume Next
	Err.clear
	
    Dim strName: strName = ""
    Dim strDescription: strDescription = ""
    Dim strLicenseFamily: strLicenseFamily = ""
    Dim strPartialProductKey: strPartialProductKey = ""
    Dim strLicenseStatus: strLicenseStatus = ""
    Dim strLicenseStatusReason: strLicenseStatusReason = ""
    Dim strEvaluationEndDate: strEvaluationEndDate = ""
    Dim strExtendedGrace: strExtendedGrace = ""
    Dim strGenuineStatus: strGenuineStatus = ""
    Dim strGracePeriodRemaining: strGracePeriodRemaining = ""
    Dim strDiscoveredKeyManagementServiceMachineName: strDiscoveredKeyManagementServiceMachineName = ""
    Dim strDiscoveredKeyManagementServiceMachinePort: strDiscoveredKeyManagementServiceMachinePort = ""
    Dim strKeyManagementServiceMachine: strKeyManagementServiceMachine = ""
    Dim strKeyManagementServicePort: strKeyManagementServicePort = ""
    Dim strApplicationID: strApplicationID = ""
    Dim strProductKeyID: strProductKeyID = ""

    ' Software license information
    WScript.Echo "------- Software license information from " & strComputer & " -------"
    
    ' Creating CSV file for Software License info	
    Dim File: Set File = CreateFile("SoftwareLicenseInfo.txt")
    File.WriteLine CSV(Array(_
        "Device","Name","Description","LicenseFamily","PartialProductKey", "LicenseStatus",_
        "LicenseStatusReason","EvaluationEndDate","ExtendedGrace","GenuineStatus",_
        "GracePeriodRemaining","DiscoveredKeyManagementServiceMachineName","DiscoveredKeyManagementServiceMachinePort",_
        "KeyManagementServiceMachine","KeyManagementServicePort","ApplicationID","ProductKeyID"))


    Dim q1: Set q1 = WMI.ExecQuery("Select * from SoftwareLicensingProduct WHERE PartialProductKey<>null AND LicenseStatus<>0")
    Dim q2: Set q2 = WMI.ExecQuery("Select * from OfficeSoftwareProtectionProduct WHERE PartialProductKey<>null AND LicenseStatus<>0")
    Dim o, q, qs: qs = Array(q1,q2)
	
    For Each q In qs
        Err.Clear
        For Each o In q
        If Err = 0 Then
	        With o
                Err.Clear
                strName                 = .Name                 : If Err <> 0 Then: strName = "":                 Err.Clear: End If
                strDescription          = .Description          : If Err <> 0 Then: strDescription = "":          Err.Clear: End If
                strLicenseFamily        = .LicenseFamily        : If Err <> 0 Then: strLicenseFamily = "":        Err.Clear: End If
                strPartialProductKey    = .PartialProductKey    : If Err <> 0 Then: strPartialProductKey = "":    Err.Clear: End If
                strLicenseStatus        = .LicenseStatus        : If Err <> 0 Then: strLicenseStatus = "":        Err.Clear: End If
                strLicenseStatusReason  = .LicenseStatusReason  : If Err <> 0 Then: strLicenseStatusReason = "":  Err.Clear: End If
                strEvaluationEndDate    = .EvaluationEndDate    : If Err <> 0 Then: strEvaluationEndDate = "":    Err.Clear: End If
                strExtendedGrace        = .ExtendedGrace        : If Err <> 0 Then: strExtendedGrace = "":        Err.Clear: End If
                strGenuineStatus        = .GenuineStatus        : If Err <> 0 Then: strGenuineStatus = "":        Err.Clear: End If
                strGracePeriodRemaining = .GracePeriodRemaining : If Err <> 0 Then: strGracePeriodRemaining = "": Err.Clear: End If
                strApplicationID        = .ApplicationID        : If Err <> 0 Then: strApplicationID = "":        Err.Clear: End If
                strProductKeyID         = .ProductKeyID         : If Err <> 0 Then: strProductKeyID = "":         Err.Clear: End If
                strKeyManagementServiceMachine = .KeyManagementServiceMachine : If Err <> 0 Then: strKeyManagementServiceMachine = "": Err.Clear: End If
                strKeyManagementServicePort    = .KeyManagementServicePort    : If Err <> 0 Then: strKeyManagementServicePort = "":    Err.Clear: End If
                strDiscoveredKeyManagementServiceMachineName = .DiscoveredKeyManagementServiceMachineName : If Err <> 0 Then: strDiscoveredKeyManagementServiceMachineName = "": Err.Clear: End If
                strDiscoveredKeyManagementServiceMachinePort = .DiscoveredKeyManagementServiceMachinePort : If Err <> 0 Then: strDiscoveredKeyManagementServiceMachinePort = "": Err.Clear: End If
            
                File.WriteLine CSV(Array(strAliasComputerName,strName,strDescription,strLicenseFamily,_
                    strPartialProductKey,strLicenseStatus,strLicenseStatusReason,_
                    strEvaluationEndDate,strExtendedGrace,strGenuineStatus,strGracePeriodRemaining,_
                    strDiscoveredKeyManagementServiceMachineName,strDiscoveredKeyManagementServiceMachinePort,_
                    strKeyManagementServiceMachine,strKeyManagementServicePort,strApplicationID,strProductKeyID))

	        End With
        End If
	    Next
    Next
	
    File.Close
	SoftwareLicenseInfo = FileOK("SoftwareLicenseInfo.txt")

End Function

'********************************************
'Purpose:
'Gather and write information about the various System Dates
Function SystemDates
	On Error Resume Next
	Err.clear
	
    Dim strBIOSDate: strBIOSDate = ""
    Dim strInstallDate: strInstallDate = ""
    Dim strLastBootUpTime: strLastBootUpTime = ""
    Dim strLocalDateTime: strLocalDateTime = ""

	WScript.Echo "------- System Date information from " & strComputer & " -------"

    With Query1(WMI, "Select * from Win32_BIOS")
		strBIOSDate = .ReleaseDate
	End With

    With Query1(WMI, "Select * from Win32_OperatingSystem")
		strInstallDate = .InstallDate
        strLastBootUpTime = .LastBootUpTime
        strLocalDateTime = .LocalDateTime
	End With

    If Len(strBIOSDate) > 8 Then
        strBIOSDate = Mid(strBIOSDate, 1, 8)
    End If
    If Len(strInstallDate) > 8 Then
        strInstallDate = Mid(strInstallDate, 1, 8)
    End If
    If Len(strLastBootUpTime) > 8 Then
        strLastBootUpTime = Mid(strLastBootUpTime, 1, 8)
    End If
    If Len(strLocalDateTime) > 8 Then
        strLocalDateTime = Mid(strLocalDateTime, 1, 8)
    End If
    
    ' Creating CSV file for System dates
    Dim File: Set File = CreateFile("SystemDates.txt")
    File.WriteLine CSV(Array("Device","BIOSDate","InstallDate","LastBootUpTime", "LocalDateTime"))

    File.WriteLine CSV(Array(strAliasComputerName, strBIOSDate, strInstallDate, strLastBootUpTime, strLocalDateTime))
	
    File.Close
	SystemDates = FileOK("SystemDates.txt")

End Function

'********************************************
'Purpose:
'Gather and write information about the various System Dates
Function ServerFeatures
	On Error Resume Next
	Err.clear
	
    Dim nADInstalled: nADInstalled = -1
    Dim nIISInstalled: nIISInstalled = -1
    Dim nRDSInstalled: nRDSInstalled = -1
    Dim nHyperVInstalled: nHyperVInstalled = -1
    Dim nClusteringInstalled: nClusteringInstalled = -1

	WScript.Echo "------- Server Feature information from " & strComputer & " -------"

    Dim o,q: Set q = WMI.ExecQuery("Select * from Win32_ServerFeature")

	For Each o In q
        If Err = 0 And nADInstalled = -1 Then
            ' This OS supports Win32_ServerFeature
            nADInstalled = 0
            nIISInstalled = 0
            nRDSInstalled = 0
            nHyperVInstalled = 0
            nClusteringInstalled = 0
        End If

	    Select Case o.ID
		    Case 2: nIISInstalled = 1
            Case 10: nADInstalled = 1
            Case 18: nRDSInstalled = 1
            Case 20: nHyperVInstalled = 1
            Case 33: nClusteringInstalled = 1
	    End Select
	Next
    
    ' Creating CSV file for Server features
    Dim File: Set File = CreateFile("ServerFeatures.txt")
    File.WriteLine CSV(Array("Device","ADInstalled","IISInstalled","RDSInstalled", "HyperVInstalled", "ClusteringInstalled"))

    File.WriteLine CSV(Array(strAliasComputerName, nADInstalled, nIISInstalled, nRDSInstalled, nHyperVInstalled, nClusteringInstalled))
	
    File.Close
	ServerFeatures = FileOK("SystemDates.txt")

End Function

'********************************************
'Purpose:
'Gather and write information about HyperV instances
Function HyperV
	On Error Resume Next
	Err.clear
	
	Call ConnectWMIVIRT()
	
	Err.clear
	Dim mgm: Set mgm = Query1(WMIVIRT, "select * from Msvm_VirtualSystemManagementService")
	
	If Err <> 0 Or IsNull(mgm) Then: HyperV = True: Exit Function: End If
	
	WScript.Echo "------- HyperV information from " & strComputer & " -------"
	
    ' Creating CSV file for HyperV instances
	Dim File: Set File = CreateFile("HyperV.txt")
    File.WriteLine CSV(Array("ComputerName", "Name", "ElementName", "NumberofProcessors", "CreationTime",_
                             "EnabledState","GuestOperatingSystem"))
	
	Dim param: Set param = mgm.Methods_("GetSummaryInformation").InParameters.SpawnInstance_()
	
	param.SettingData = Nothing
	param.RequestedInformation = Array(0,1,2,4,100,106)
	
	Dim out: Set out = mgm.ExecMethod_("GetSummaryInformation", param)
	
	If out.ReturnValue = 0 Then
		Dim info: For Each info In out.SummaryInformation
		With info
			Dim enabled
	
			Select Case .EnabledState
				Case 2: enabled = "On"
				Case 3: enabled = "Off"
				Case Else: enabled = "" & .EnabledState
			End Select
	
			File.WriteLine CSV(Array(strAliasComputerName, .Name, .ElementName, .NumberofProcessors, CutDate(.CreationTime), enabled, .GuestOperatingSystem))
		End With
		Next
	End If
	
	File.Close
	HyperV = True 'FileOK("HyperV.txt")

    WScript.Echo "------- HyperV information from " & strComputer & " (2) -------"
    RunHyperVPSScript(strAliasComputerName)

End Function
'********************************************
'Purpose:
'Gather and write information about user logons based on their locally installed profiles
Function Computer_Logons
	On Error Resume Next
	Err.clear
	
	WScript.Echo "------- User logon Information from "& strComputer & " -------"
		
    ' Creating CSV file for user logons
	Dim File: Set File = CreateFile("UserLogons.txt")
    File.WriteLine CSV(Array("ComputerName", "FileName", "CreationDate", "LastModified", "LastAccessed"))

	' Get profile directory from registry and split into path and drive
	Dim profiles: Call REG.GetStringValue(HKLM, "SOFTWARE\MICROSOFT\WINDOWS NT\CURRENTVERSION\PROFILELIST", "PROFILESDIRECTORY", profiles)
	Dim path: path = Replace(Replace(Right(profiles,Len(profiles)-2)  & "\","\","\\"),"\\\","\\")
	Dim drive: drive = Left(profiles,2)
	
	' Skip network locations
	If Not (InStr(profiles, "\\") > 0) Then
		Dim o,q: Set q = WMI.ExecQuery("Select * from Win32_Directory Where Hidden=false And Path='" & path & "' And drive='" & drive & "'")
		For Each o In q
		With o
			If UCase(.FileName) <> "ALL USERS" Then
				File.WriteLine CSV(Array(strAliasComputerName, .FileName, CutDate(.CreationDate), CutDate(.LastModified), CutDate(.LastAccessed)))
			End If
		End With
		Next
	End If
	
	File.Close
	Computer_Logons = FileOK("UserLogons.txt")

End Function
'********************************************
'Purpose:
'Gather and write information about running processes and call hook function for extra processing
Function UserProfiles
	On Error Resume Next
	Err.clear
	
	WScript.Echo "------- User Profile Information from " & strComputer & " -------"	
	
    Dim strLastUseTime: strLastUseTime = ""
	Dim File: Set File = CreateFile("UserProfiles.txt")
    File.WriteLine CSV(Array("Computer", "SID", "LocalPath", "Special", "RoamingConfigured", "LastUseTime"))
    	
	Dim o,q: Set q = WMI.ExecQuery("Select * from Win32_UserProfile")
	For Each o In q
	With o
        strLastUseTime = .LastUseTime
        If Len(strLastUseTime) > 8 Then
            strLastUseTime = Mid(strLastUseTime, 1, 8)
        End If
		File.WriteLine CSV(Array(strAliasComputerName, .SID, .LocalPath, .Special, .RoamingConfigured, strLastUseTime))
	End With
	Next
	
	File.Close
	UserProfiles = FileOK("UserProfiles.txt")

End Function
'********************************************
'Purpose:
'Gather and write information about computer access groups (Remote Desktop and Administrators)
Function Computer_Access
	On Error Resume Next
	Err.clear
	
	WScript.Echo "------- Access group Information from "& strComputer & " -------"
	
    ' Creating CSV file for computer access groups
	Dim File: Set File = CreateFile("AccessGroup.txt")
    File.WriteLine CSV(Array("ComputerName", "GroupName", "Member"))
		
	Dim objAUser, AdministratorGroup: Set AdministratorGroup = GetObject("WinNT://" & strComputer & "/Administrators")
	For Each objAUser in AdministratorGroup.Members
		File.WriteLine CSV(Array(strAliasComputerName, AdministratorGroup.Name, Replace(objAUser.ADsPath,"WinNT://","")))
	Next
	
	Dim objRUser, RDPGroup: Set RDPGroup = GetObject("WinNT://" & strComputer & "/Remote Desktop Users,group")
	For Each objRUser in RDPGroup.Members
		File.WriteLine CSV(Array(strAliasComputerName, RDPGroup.Name, Replace(objRUser.ADsPath,"WinNT://","")))
	Next

    Dim objLocalUser, LocalUserGroup: Set LocalUserGroup = GetObject("WinNT://" & strComputer & "/Users,group")
	For Each objLocalUser in LocalUserGroup.Members
		File.WriteLine CSV(Array(strAliasComputerName, LocalUserGroup.Name, Replace(objLocalUser.ADsPath,"WinNT://","")))
	Next
	
	File.Close
	
	Computer_Access = FileOK("AccessGroup.txt")
End Function
'*******************************************************************************
'Date conversion
'Purpose:
'Converts a WMI formatted date to a VBScript Date object
Function Integer8Date(d)
    Integer8Date = CDate(#1/1/1601# + (((d.HighPart * (2 ^ 32)) + d.LowPart) / 600000000 ) / 1440)
End Function

'Purpose:
'Converts a 'yyyymmdd hh:mm:ss' formatted date to 'yyyymmdd'
Function CutDate(d)
	CutDate = Left(d,8) '& Mid(d,5,2) & Mid(d,7,2)
End function

'********************************************
'Purpose:
'Gather and write information from Active Directory about computers in the domain
Sub AD_Devices
	On Error Resume Next
	Err.clear
	
	If Not bADDomainFound Then: WScript.Quit: End If

    ' Call CSVDE
    RunCSVDE()

    ' Call AD PowerShell script
    RunActiveDirectoryPSScript()
	
	' Temporary variables
	Dim line, adoRS
	
	' ADODB connection
	Dim adoConn: Set adoConn = CreateObject("ADODB.Connection")
	Dim adoCmd:  Set adoCmd = CreateObject("ADODB.Command")
	adoConn.Provider = "ADsDSOObject"
	adoConn.Open "Active Directory Provider"
	Set adoCmd.ActiveConnection = adoConn
	
	' Domain listing
	Set adoRS = adoConn.Execute("<GC://" & DNC & ">;(objectcategory=domainDNS);name;SubTree")
	line = ""
	Do Until adoRS.EOF
		line = line & CSV(Array(adoRS.Fields("Name").Value))
		adoRS.MoveNext
	Loop
	adoRS.Close
	
    ' Creating CSV file to get info from Active Directory about computers in the domain
	Dim FileFull: Set FileFull = FSO.CreateTextFile(FullPath & "EY_ADComputers.txt", True, True)
    
    ' Creating CSV file to get info from Active Directory about servers in the domain
	Dim File:     Set File     = FSO.CreateTextFile(FullPath & "DeviceList.txt", True, True)
	
	FileFull.WriteLine "### Currentdomain=" & DNC
	FileFull.WriteLine "### Full domainlist=" & line

	FileFull.WriteLine CSV(Array("Name", "PasswordAge", "OS", "OS_Version", "WhenCreated", "DN", "LastLogon", "LastLogonTimestamp"))
	
	' Server listing
	adoCmd.CommandText = "<LDAP://" & DNC & ">;(objectCategory=computer);Name,operatingSystem,operatingSystemVersion,pwdLastSet,whenCreated,distinguishedName,lastLogon,lastLogonTimestamp;subtree"
	adoCmd.Properties("Page Size") = 500
	adoCmd.Properties("Timeout") = 25
	adoCmd.Properties("Cache Results") = False
	Set adoRS = adoCmd.Execute
	Do Until adoRS.EOF
		Dim strName:  strName  = UCase(Trim(adoRS.Fields("Name").Value))
		Dim strOS:    strOS    = Trim(adoRS.Fields("operatingSystem").Value)
		Dim strOSver: strOSver = Trim(adoRS.Fields("operatingSystemVersion").Value)
		Dim age:      age      = DateDiff("d", Integer8Date(adoRS.Fields("pwdLastSet").Value), Now)
		Dim created:  created  = adoRS.Fields("whenCreated").Value
		Dim dn:       dn       = adoRS.Fields("distinguishedName").Value
		Dim logon:    logon    = Integer8Date(adoRS.Fields("lastLogon").Value)
		Dim logonts:  logonts  = Integer8Date(adoRS.Fields("lastLogonTimestamp").Value)
				
		line = CSV(Array(strName,age,strOS,strOSver,created,dn,logon,logonts))
		
		FileFull.WriteLine line
		
		If age <= ageLimit Then
			If Instr(UCase(strOS),"SERVER") > 0 Then
				File.WriteLine line
			Else
				If Not bOnlyServersFromAD Then
					File.WriteLine line 
				End If
			End If
		End If
		adoRS.MoveNext
	Loop
	adoRS.Close
	
	FileFull.Close
	File.Close
	
	Wscript.Echo "*** DeviceList.txt and EY_ADComputers.txt Created ***"
End Sub
'********************************************
Dim AD_UserInfoDict: Set AD_UserInfoDict = Nothing

'Purpose:
'Gathers information about a specific AD user
Function AD_UserInfo(path)
	On Error Resume Next
	Err.clear
	
	If AD_UserInfoDict Is Nothing Then: Set AD_UserInfoDict = CreateObject("Scripting.Dictionary"): End If

	If AD_UserInfoDict.Exists(path) Then
		AD_UserInfo = AD_UserInfoDict.Item(path)
		Exit Function
	End If
	
	'UserFlags	
	Const ADS_UF_SCRIPT = &H0001 
	Const ADS_UF_ACCOUNTDISABLE = &H0002 
	Const ADS_UF_HOMEDIR_REQUIRED = &H0008 
	Const ADS_UF_LOCKOUT = &H0010 
	Const ADS_UF_PASSWD_NOTREQD = &H0020 
	Const ADS_UF_PASSWD_CANT_CHANGE = &H0040 
	Const ADS_UF_ENCRYPTED_TEXT_PASSWORD_ALLOWED = &H0080 
	Const ADS_UF_DONT_EXPIRE_PASSWD = &H10000 
	Const ADS_UF_SMARTCARD_REQUIRED = &H40000 
	Const ADS_UF_PASSWORD_EXPIRED = &H800000
	
	With GetObject("LDAP://" & path)
		Dim strPwdLastSet, strLastLogon, bPwdRequired, strDisabled, strPwdExpired, strLockOut
		
		If .PasswordLastChanged = "" Then: strPwdLastSet = "Never": Else strPwdLastSet = .PasswordLastChanged: End If
		strLastLogon = Integer8Date(.Get("lastLogonTimestamp"))
		
		Dim strUserName, strDescription, strDisplayName, strGivenName, strSirName, strDN, strUPN

		strUserName    = .sAMAccountName
		strDescription = .Description
		strDisplayName = .DisplayName
		strGivenName   = .GivenName
		strSirName     = .SN
		strDN          = .DistinguishedName
		strUPN 		   = .userPrincipalName
		
			
		AD_UserInfo = CSV(Array(strUserName, strDisplayName, strGivenName, strUPN, strSirName, strPwdLastSet, _
			strLastLogon, .WhenCreated, .WhenChanged))
	End With
	
	Call AD_UserInfoDict.Add(path, AD_UserInfo)
End Function
'********************************************
'Purpose:
'Gathers and writes information about all AD users in current domain
Sub AD_Users
	On Error Resume Next
	Err.clear
	
	If Not bADDomainFound Then: WScript.Quit: End If
	
	WScript.Echo "------- Listing AD Users From " & DNC & " -------"
	
    ' Creating CSV file for all AD users in current domain
	Dim File: Set File = FSO.CreateTextFile(OutputPath & "#EY_ADUsers.txt", True, True)	
	File.WriteLine CSV(Array("UserName", "DisplayName", "GivenName", "UserPrincipalName", "SirName", _
							 "PwdLastSet", "LastLogonTimeStamp", "WhenCreated", "WhenChanged"))
	
	' ADODB connection
	Dim adoConn: Set adoConn = CreateObject("ADODB.Connection")
	Dim adoCmd:  Set adoCmd = CreateObject("ADODB.Command")
	adoConn.Provider = "ADsDSOObject"
	adoConn.Open "Active Directory Provider"
	Set adoCmd.ActiveConnection = adoConn
	adoCmd.CommandText = "<LDAP://" & DNC & ">;(&(objectCategory=person)(objectClass=user));distinguishedName;subtree"
	adoCmd.Properties("Page Size") = 1000
	adoCmd.Properties("Timeout") = 25
	adoCmd.Properties("Cache Results") = False
	Dim adoRS: Set adoRS = adoCmd.Execute
	
	' Users
	Do Until adoRS.EOF
		File.WriteLine AD_UserInfo(adoRS.Fields("DistinguishedName").Value)
		adoRS.MoveNext
	Loop
	adoRS.Close
	
	File.Close
	
End Sub
'********************************************
'Purpose:
'Gathers information about a specific AD group
Function AD_GroupInfo(path)
	On Error Resume Next
	Err.clear
	
	Const GROUP_TYPE_BUILTIN_LOCAL_GROUP= &h00000001
	Const GROUP_TYPE_ACCOUNT_GROUP 		= &h00000002
	Const GROUP_TYPE_RESOURCE_GROUP 	= &h00000004
	Const GROUP_TYPE_UNIVERSAL_GROUP 	= &h00000008
	Const GROUP_TYPE_SECURITY_ENABLED	= &h80000000
	
	With GetObject("LDAP://" & path)
		Dim strGroupType: strGroupType = ""
		
		If .GroupType AND GROUP_TYPE_BUILTIN_LOCAL_GROUP Then: strGroupType = strGroupType & "Builtin Local,": End If
		If .GroupType AND GROUP_TYPE_ACCOUNT_GROUP       Then: strGroupType = strGroupType & "GlobalGroup,": End If
		If .GroupType AND GROUP_TYPE_RESOURCE_GROUP      Then: strGroupType = strGroupType & "Domain Local,": End If
		If .GroupType AND GROUP_TYPE_UNIVERSAL_GROUP     Then: strGroupType = strGroupType & "Universal": End If
		If .GroupType AND GROUP_TYPE_SECURITY_ENABLED    Then: strGroupType = strGroupType & "security-enabled group,": End If
		
		AD_GroupInfo = CSV(Array(.sAMAccountName, .Description, .WhenCreated, .WhenChanged))
	End With
End Function
'********************************************
'Purpose:
'Gathers and writes information about a specific AD group and its members (with nesting)
Sub AD_GroupMembers(path, groups, strGroupInfo, File)
	On Error Resume Next
	Err.clear
	
	' Group members
	Dim o, q: Set q = GetObject("LDAP://" & path)
	For Each o In q.Members
	With o
		' Nested group
		If UCase(.Class) = "GROUP" Then
			Dim ADsPath: ADsPath = Replace(.ADsPath,"LDAP://","")
			' Follow if not seen before
			If Not groups.Exists(ADsPath) Then
				groups.Add ADsPath, 1
				Call AD_GroupMembers(ADsPath, groups, strGroupInfo, File)
			End IF
		' User
		ElseIf UCase(.Class) = "USER" Then
			File.WriteLine strGroupInfo & AD_UserInfo(.DistinguishedName)
		End If
	End With
	Next
End Sub
'********************************************
'Purpose:
'Gathers and writes information about all AD group and their members in the domain
Sub AD_Groups
	On Error Resume Next
	Err.clear
	
	'If Instr(UCase(strDomainRole),"Standalone") > 0 Then
	'	WScript.Echo "Not listing domain groups, because " & strComputer & " is a " & strDomainRole
	'	Exit Sub
	'End If
	
	If Not bADDomainFound Then: WScript.Quit: End If
	
	WScript.Echo "------- Listing AD Groups From " & DNC & " -------"
	
    ' Creating CSV file for all AD group
	Dim File: Set File = CreateFile("#EY_ADGroups.txt")
	File.WriteLine CSV(Array("GroupName", "GroupDescription", "GroupWhenCreated", "GroupWhenChanged", _
                             "UserName", "DisplayName", "GivenName", "Surname", _
							 "PwdLastSet", "LastLogonTimeStamp", "WhenCreated", "WhenChanged"))
	
	' ADODB connection
	Dim adoConn: Set adoConn = CreateObject("ADODB.Connection")
	Dim adoCmd:  Set adoCmd = CreateObject("ADODB.Command")
	adoConn.Provider = "ADsDSOObject"
	adoConn.Open "Active Directory Provider"
	Set adoCmd.ActiveConnection = adoConn
	adoCmd.CommandText = "<LDAP://" & DNC & ">;(&(objectCategory=Group));distinguishedName;subtree"
	adoCmd.Properties("Page Size") = 1000
	adoCmd.Properties("Timeout") = 25
	adoCmd.Properties("Cache Results") = False
	Dim adoRS: Set adoRS = adoCmd.Execute
	
	' Groups
	Dim groups: set groups = CreateObject("Scripting.Dictionary")
	Do Until adoRS.EOF
		Dim strDN: strDN = adoRS.Fields("DistinguishedName").Value
		Dim strGI: strGI = AD_GroupInfo(strDN)
		
		
		groups.RemoveAll
		groups.Add strDN, 1
		Call AD_GroupMembers(strDN, groups, strGI, File)
		adoRS.MoveNext
	Loop
	adoRS.Close
	
	File.Close
End Sub
'********************************************
'********************************************
'Purpose:
'Performs a network ping to determine if network machine is online
Function Ping(address)
	On Error Resume Next
	Err.clear
	
	If bLocalMode Then: Ping = True: Exit Function: End If
	
	With Query1(GetObject("winmgmts:{impersonationLevel=impersonate}"), "Select * from Win32_PingStatus Where address='" & address & "'")
		If .StatusCode = 0 Then: Ping = True: Else: Ping = False: End If
	End With
End Function
'********************************************
'Purpose:
'Loads the name part "DeviceList.txt' file into memory and makes a copy of 'DeviceList.txt' and 'EY_ADComputers.txt' into output directory
Function Load_Devices
	On Error Resume Next
	Err.clear
	
	Dim filenames: filenames = Array(FullPath & "DeviceList.txt", _
									 FullPath & "EY_ADComputers.txt")
	Dim arr()
	
	If Not FSO.FileExists(filenames(0)) Then
		WScript.Echo "********** DeviceList.txt not found!"
		WScript.Quit
	End If
	
	Dim File: Set File = FSO.OpenTextFile(filenames(0), 1, False, -2) ' Possibly force unicode with -1
	Dim line, idx, i: i = 0
	
	Do While File.AtEndOfStream <> True
		line = Trim(File.ReadLine())
		'idx = InStr(line, ";")
		idx = InStr(line, chr(9))
		If idx > 1 Then
			line = Left(line, idx-1)
		End If
		line = Trim(Replace(Replace(Replace(line, """", ""), vbCr, ""), vbLf, ""))
		If Len(line) > 0 Then
			ReDim Preserve arr(i)
			arr(i) = UCase(line)
			i = i+1
		End If
	Loop
	
	If i > 0 Then
		WScript.Echo "Read " & i & " systems into memory"
		
		' Copy DeviceList files to output directory
		Call FSO.CopyFile(filenames(0), OutputPath & "#DeviceList.txt", True)
		Call FSO.CopyFile(filenames(1), OutputPath & "#EY_ADComputers.txt", True)
	Else
		WScript.Echo "DeviceList.txt is empty"
        WScript.Echo "No devices will be scanned"
		WScript.Quit
	End If
	
	Load_Devices = arr
	
	File.Close
End Function

'********************************************
'Purpose:
'Hook function for application specific processing - called from "software"
Sub Software_Hook(name, hive, path, File)
	On Error Resume Next
	Err.clear
	
	Dim uname: uname = Trim(UCase(name))
	
	' Disregard empty lines
	If uname = "" Then: Exit Sub: End If
	
	
	If InStr(uname, "MICROSOFT ISA SERVER") > 0 Or InStr(uname, "MICROSOFT FOREFRONT") > 0 Then
		Call Software_ISA_ForeFront(hive, path)
	End If

	If InStr(uname, "MICROSOFT EXCHANGE") > 0 Then
		Call Software_Exchange(hive, path)
	End If

	If InStr(uname, "BIZTALK") > 0 Then
		Call Software_Biztalk(hive, path)
	End If

End Sub
'********************************************
'********************************************
'********************************************
'Purpose:
'Hook function for application specific processing - called from "services"
Sub Services_Hook(name, dname, path, state)
	On Error Resume Next
	Err.clear
	
	' Protect against blank names
	If Trim(name) = ""  Then: name  = "<blank>": End If
	If Trim(dname) = "" Then: dname = "<blank>": End If
	
	' Default SQL instance
	If UCase(name) = "MSSQLSERVER" Then: name = "MSSQL$" & name: End If

	' All SQL instances
	   ' "MSSQL$" - DataBase engine
       ' "REPORTSERVER" - Reporting Services
	   ' "MSOLAP$" - Analysis Services
	   ' "MSSQLSERVEROLAPSERVICE" - Analysis Services
	   ' "MSDTSSERVER" - Integration Services
	   
	If InStr(UCase(name), "MSSQL$") > 0 or InStr(UCase(name), "REPORTSERVER") > 0 or InStr(UCase(name), "MSOLAP$") > 0 or InStr(UCase(name), "MSSQLSERVEROLAPSERVICE") > 0 or InStr(UCase(name), "MSDTSSERVER") > 0 Then ' Integration Services
		Call Software_MSSQL(UCase(name), state)
	End If

	' SharePoint 2010
	If InStr(UCase(name), "SPADMIN") > 0 Then
		Call Software_SharePoint()
        Call RunSharePointPSScript(strComputer, True)
	End If
	
	' Cluster
	If InStr(UCase(dname), "CLUSTER SERVICE") > 0 Then
		Call Services_Cluster()
	End If

End Sub
'********************************************
Dim bServices_Cluster: bServices_Cluster = False

'Purpose:
'Gathers and writes information about the Microsoft Clustering Service
Sub Services_Cluster()
	On Error Resume Next
	Err.clear
	
	' Only check cluster once
	If bServices_Cluster Then: Exit Sub: End If
	bServices_Cluster = True
	
	WScript.Echo "------- Cluster service on " & strComputer
	
    ' Creating CSV file for Microsoft Clustering Service
	Dim File: Set File = CreateFile("Cluster.txt")
    File.WriteLine CSV(Array("ComputerName", "ClusterAlias", "ClusterMode", "GroupComponent", "PartComponent"))
			
	' Connect to WMI Cluster service
	Call ConnectWMICLUS()
	
	Err.clear
	Dim o,q: Set q = WMICLUS.ExecQuery("Select * from MSCluster_Cluster")
	If Err = 0 Then
		Dim active: Set active = CreateObject("Scripting.Dictionary")
		Dim strPC, strGC
		Dim strClusterAlias: strClusterAlias = ""
		
		' Cluster alias
		For Each o In q
			strClusterAlias = o.Name
		Next
		
		' Active nodes
		Set q = WMICLUS.ExecQuery("select * from MSCluster_NodeToActiveResource")
		For Each o In q
		With o
			strGC = Replace(Replace(.GroupComponent, "MSCluster_Node.Name=",""),"""","")
			strPC = Replace(Replace(.PartComponent, "MSCluster_Resource.Name=",""),"""","")
			
			File.WriteLine CSV(Array(strAliasComputerName, strClusterAlias, "ACTIVE", strGC, strPC))
			Call active.Add(CSV(Array(strGC, strPC)), 1)
		End With
		Next
		
		' All nodes
		Set q = WMICLUS.ExecQuery("select * from MSCluster_ResourceToPossibleOwner")
		For Each o In q
		With o
			' For unknown reasons these values are switched around
			strGC = Replace(Replace(.GroupComponent, "MSCluster_Resource.Name=",""),"""","")
			strPC = Replace(Replace(.PartComponent, "MSCluster_Node.Name=",""),"""","")
			
			' Node is passive if not already found
			If active.Item(CSV(Array(strPC, strGC))) <> 1 Then
				File.WriteLine CSV(Array(strAliasComputerName, strClusterAlias, "PASSIVE", strPC, strGC))
			End If
		End With
		Next
	End If
	
	File.Close
End Sub
'********************************************
Dim bSoftware_Biztalk: bSoftware_Biztalk = False

'Purpose:
'Gathers and writes information about Microsoft Biztalk
Sub Software_Biztalk(hive, path)
	On Error Resume Next
	Err.clear
	
	If bSoftware_Biztalk Then: Exit Sub: End If
	
	bSoftware_Biztalk = True
	
	WScript.Echo "------- BizTalk on " & strComputer
	
    ' Creating CSV file for Microsoft Biztalk
	Dim File: Set File = CreateFile("BizTalk.txt")
    File.WriteLine CSV(Array("ComputerName", "InstallDate", "ProductEdition", "ProductVersion", "ProductName", "FoundInRegistry"))
			
	Dim vals(5),wow,key,keys
	
	vals(0) = strComputer
	vals(5) = "Not Found"
	
	Call REG.GetStringValue(HKLM,"SOFTWARE\Microsoft\BizTalk Server\3.0","InstallDate",vals(1))
	Call REG.GetStringValue(HKLM,"SOFTWARE\Microsoft\BizTalk Server\3.0","ProductEdition",vals(2))
	Call REG.GetStringValue(HKLM,"SOFTWARE\Microsoft\BizTalk Server\3.0","ProductVersion",vals(3))
	Call REG.GetStringValue(HKLM,"SOFTWARE\Microsoft\BizTalk Server\3.0","ProductName",vals(4))
	
	For Each wow In Array("", "Wow6432Node\")
		Call REG.EnumKey(HKLM,"SOFTWARE\" & wow & "Microsoft\BizTalk Server\3.0\ConfigFramework\ConfigModules\MSEBiz.RuleEngineConfig",keys)
		If IsArray(keys) Then
			For Each key In keys
				If UCase(key) = "RULESENGINE" Then
					vals(5) = "Found"
					Exit For
				End If
			Next
		End If
	Next
	
	File.WriteLine CSV(vals)
	
	File.Close
End Sub
'********************************************
'Purpose:
'Gathers and writes information about which server and database are backend for Microsoft Sharepoint
Sub Software_SharePoint()
	On Error Resume Next
	Err.clear
	
	WScript.Echo "------- SharePoint on " & strComputer
	
    ' Creating CSV file for server and database are backend for Microsoft Sharepoint
	Dim File: Set File = CreateFile("SharePoint.txt")
    File.WriteLine CSV(Array("ComputerName", "DataSource", "InitialCatalog"))
		
	Dim ds,ic,val,vals,key,keys
	
	Call REG.EnumKey(HKLM,"SOFTWARE\Microsoft\Shared Tools\Web Server Extensions",keys)
	For Each key In keys
		If REG.GetStringValue(HKLM,"SOFTWARE\Microsoft\Shared Tools\Web Server Extensions\" & key & "\Secure\ConfigDB","dsn",val) = 0 Then
			ds = "": ic = ""
			vals = Split(val,";")
			For Each val In vals
				If InStr(val, "Data Source=") > 0 Then: ds = Replace(val,"Data Source=",""): End If
				If InStr(val, "Initial Catalog=") > 0 Then: ic = Replace(val,"Initial Catalog=",""): End If
			Next
			File.WriteLine CSV(Array(strAliasComputerName,ds,ic))
		End If
	Next
	
	File.Close
	FileOKOrDelete("SharePoint.txt")

End Sub
'********************************************
Dim bSoftware_ISA_ForeFront: bSoftware_ISA_ForeFront = False

'Purpose:
'Gathers and writes information about Microsoft ISA and Microsoft ForeFront
Sub Software_ISA_ForeFront(hive, path)
	On Error Resume Next
	Dim version, major, name, isa_edition, edition, x
	
	WScript.Echo "------- ISA/ForeFront Server on " & strComputer
		
    ' Creating CSV file for Microsoft ISA and Microsoft ForeFront
	Dim File: Set File = CreateOrOpenFile("ISA.txt", bSoftware_ISA_ForeFront)
    if (FileIsEmpty("ISA.txt")) Then
        File.WriteLine CSV(Array("ComputerName", "Name", "Edition", "Version"))	
    End if	
	
	bSoftware_ISA_ForeFront = True
	
	edition = "Unknown"
	isa_edition = "Standard"
	
	' Version detection for ISA 2004 + ISA 2006
	If REG.EnumKey(hive,"IsaStg_Eff2",x) = 0 Then
		isa_edition = "Enterprise"
	End If
	
	Call REG.GetStringValue(hive, path, "DisplayName", name)
	Call REG.GetDWORDValue (hive, path, "VersionMajor", major)
	Call REG.GetStringValue(hive, path, "DisplayVersion", version)
	
	' ISA Server 2004+2006 bug
	If InStr(UCASE(name), "SQL SERVER") > 0 Then: Exit Sub: End If

	Select Case major
		' TODO:
		' 3 => ISA 2000
		' 6 => TMG Medium Business Edition
		Case 4: edition = isa_edition ' ISA 2004
		Case 5: edition = isa_edition ' ISA 2006
		Case 7: ' ForeFront
			If InStr(name, "Gateway EE") > 0 Then
				edition = "Enterprise"
			ElseIf InStr(name, "Gateway SE") > 0 Then
				edition = "Standard"
			End If
	End Select

	File.WriteLine CSV(Array(strAliasComputerName, name, edition, version))
	File.Close
End Sub
'********************************************
Dim bSoftware_Exchange: bSoftware_Exchange = False

'Purpose:
'Gathers and writes information about Microsoft Exchange 2000-2003
Sub Software_Exchange(hive, path)
	On Error Resume Next
	
	WScript.Echo "------- Exchange Server on " & strComputer
	
    ' Creating CSV file for Exchange 2000-2003
	Dim File: Set File = CreateOrOpenFile("Exchange.txt", bSoftware_Exchange)
    if (FileIsEmpty("Exchange.txt")) Then
        File.WriteLine CSV(Array("ComputerName", "Name", "Edition", "Version"))	
    End If	
	
	Dim name, version, edition: name = "": version = "": edition =""
	
	bSoftware_Exchange = True
		
	Dim guids: guids = Array( _
		Array("DB20F7FD-67BC-4813-8808-78F63E89EB56", "2000", "Standard Version Upgrade Package (VUP)"), _
		Array("775CF3DA-C007-4709-B4CC-CE2239BE2E03", "2000", "Standard Full Packaged Product (FPP)"), _
		Array("FC6FA539-452D-4a9b-8065-C1FA74B86F83", "2000", "Standard Evaluation 120 days (EVAL)"), _
		Array("D3574E0C-360A-44d5-858C-33323C2D79F2", "2000", "Enterprise Version Upgrade Package (VUP"), _
		Array("F8567801-906B-439b-8D6A-87BDFEC9BA52", "2000", "Enterprise Product Upgrade Package (PUP)"), _
		Array("65D9643D-06E8-47d6-865E-80F4CC9BB879", "2000", "Enterprise Full Packaged Product (FPP)"), _
		Array("8B102332-6052-4af3-ADFA-35A3DED0506A", "2000", "Enterprise Evaluation 120 days (EVAL)"), _
		Array("EAE76D62-2691-4883-8BBB-1F2EE6D370D5", "2003", "Standard Version Upgrade Package (VUP)"), _
		Array("9161A261-6ABE-4668-BBFA-AD06B3F642CF", "2003", "Standard Full Packaged Product (FPP)"), _
		Array("C160866F-DE53-434f-ADF1-CC42ABBF8778", "2003", "Standard Evaluation 120 days (EVAL)"), _
		Array("74F3BB3C-A434-48fa-AAC1-3FC37CD2B0DB", "2003", "Enterprise Version Upgrade Package (VUP)"), _
		Array("7F4210A8-D3B4-480a-835E-D5DAA0D0B157", "2003", "Enterprise Product Upgrade Package (PUP)"), _
		Array("F95DE19F-CF69-4b03-81B6-9EC050D20D3B", "2003", "Enterprise Full Packaged Product (FPP)"), _
		Array("2B8EC4BD-5638-47e2-8817-1A50B38A828D", "2003", "Enterprise Evaluation 120 days (EVAL)"))

	Dim arr: For Each arr In guids
		If InStr(UCase(path), UCase(arr(0))) > 0 Then
			name = "Microsoft Exchange " & arr(1)
			edition = arr(2)
			Exit For
		End If
	Next
	
	If Name<>"" Then
		File.WriteLine CSV(Array(strAliasComputerName, name, edition, version))
	End If
	File.Close
End Sub
'********************************************
Dim bSoftware_MSSQL_SharePoint: bSoftware_MSSQL_SharePoint = False

'Purpose:
'Gathers and writes information about Microsoft Sharepoint by inspection of the underlying Microsoft SQL Server database
Sub Software_MSSQL_SharePoint(instance, strDB, AC, strHost)
	On Error Resume Next
	Err.clear
	
    ' Creating CSV file for Microsoft Sharepoint - Site Features
	Dim FileSiteFeatures: Set FileSiteFeatures = CreateOrOpenFile("SQL_Sharepoint_SiteFeatures.txt", bSoftware_MSSQL_SharePoint)
    if (FileIsEmpty("SQL_Sharepoint_SiteFeatures.txt")) Then
        File.WriteLine CSV(Array("ComputerName", "Instance", "DB", "Host", "FeatureId", "TimeActivated", "Title",_
                             "FullUrl", "SiteId"))
    End If

    ' Creating CSV file for Microsoft Sharepoint - Sites
	Dim FileSites: Set FileSites = CreateOrOpenFile("SQL_Sharepoint_Sites.txt", bSoftware_MSSQL_SharePoint)
    if (FileIsEmpty("SQL_Sharepoint_Sites.txt")) Then
        File.WriteLine CSV(Array("ComputerName", "Instance", "DB", "Host", "FullUrl", "Title", "SiteId",_
                             "Id", "ParentWebId"))
    End If

    ' Creating CSV file for Microsoft Sharepoint - Users
	Dim FileUsers: Set FileUsers = CreateOrOpenFile("SQL_Sharepoint_Users.txt", bSoftware_MSSQL_SharePoint)
    if (FileIsEmpty("SQL_Sharepoint_Users.txt")) Then
        File.WriteLine CSV(Array("ComputerName", "Instance", "DB", "Host", "SiteId", "Id", "Deleted",_
                             "SiteAdmin", "IsActive", "Login", "Title"))
    End If		
	 
    ' Creating CSV file for Microsoft Sharepoint - Config
	Dim FileConfig: Set FileConfig = CreateOrOpenFile("SQL_Sharepoint_Config.txt", bSoftware_MSSQL_SharePoint)
    if (FileIsEmpty("SQL_Sharepoint_Config.txt")) Then
        File.WriteLine CSV(Array("ComputerName", "Instance", "DB", "Host", "ServerId", "Product", "Version"))
    End If
    
	Dim RS
	
	If Not bSoftware_MSSQL_SharePoint Then
		bSoftware_MSSQL_SharePoint = True
	End If
		
	' Website DBs
	If InStr(strDB, "WSS_Content") > 0 Then
		WScript.Echo "------- SharePoint from SQL connection " & strDB
		Set RS = CreateObject("ADODB.Recordset")
		
		' Site Features (2007 + 2010)
		Err.clear
		Call RS.Open("SELECT t1.[FeatureId],t1.[TimeActivated],t3.[Title],t3.[FullUrl],t1.[SiteId] FROM [" & strDB & "].[dbo].[Features] AS t1 INNER JOIN "&strDB&".[dbo].[Webs] AS t3 ON t1.WebId=t3.Id ORDER BY t1.[FeatureId],t1.[SiteId],t1.[WebId]", AC)
		If Err = 0 Then
			Do Until RS.EOF
			With RS.Fields
				FileSiteFeatures.WriteLine CSV(Array(strAliasComputerName, instance, strDB, strHost, .Item("FeatureId"), .Item("TimeActivated"), .Item("Title"), .Item("FullUrl"), .Item("SiteId")))
				RS.MoveNext
			End With
			Loop
			RS.Close
		End If
		Err.clear
		
		' Sites (2003? + 2007 + 2010)
		Call RS.Open("SELECT [FullUrl],[Title],[SiteId],[Id],[ParentWebId] FROM [" & strDB & "].[dbo].[Webs]", AC)
		If Err = 0 Then
			Do Until RS.EOF
			With RS.Fields
				FileSites.WriteLine CSV(Array(strAliasComputerName, instance, strDB, strHost, .Item("FullUrl"), .Item("Title"), .Item("SiteId"), .Item("Id"), .Item("ParentWebId")))
				RS.MoveNext
			End With
			Loop
			RS.Close
		End If
		Err.clear

		' Users (2003? + 2007 + 2010)
		Call RS.Open("SELECT [tp_SiteID],[tp_ID],[tp_Deleted],[tp_SiteAdmin],[tp_IsActive],[tp_Login],[tp_Title] FROM [" & strDB & "].[dbo].[UserInfo] ORDER BY [tp_SiteID], [tp_Login]", AC)
		If Err = 0 Then
			Do Until RS.EOF
			With RS.Fields
				FileUsers.WriteLine CSV(Array(strAliasComputerName, instance, strDB, strHost, .Item("tp_SiteID"), .Item("tp_ID"), .Item("tp_Deleted"), .Item("tp_SiteAdmin"), .Item("tp_IsActive"), .Item("tp_Login"), .Item("tp_Title")))
				RS.MoveNext
			End With
			Loop
			RS.Close
		End If
		Err.clear
	
	' Configuration DB (2010)
	Else
		' Version information
		Set RS = CreateObject("ADODB.Recordset")
		Err.clear
		Call RS.Open("SELECT DISTINCT [ServerId],[Product],[Version] FROM [" & strDB & "].[dbo].[ServerVersionInformation]", AC)
		If Err = 0 Then
			Do Until RS.EOF
			With RS.Fields
				If InStr(UCase(.Item("Product")), "SHAREPOINT") > 0 Then
					FileConfig.WriteLine CSV(Array(strAliasComputerName, instance, strDB, strHost, .Item("ServerId"), .Item("Product"), .Item("Version")))
				End If
				RS.MoveNext
			End With
			Loop
			RS.Close
		End If
		Err.clear
	End If
	
	FileSiteFeatures.Close
	FileSites.Close
	FileUsers.Close
	FileConfig.Close
End Sub
'********************************************
'********************************************
'Purpose:
'Hook function for inspection of Microsoft SQL Server databases - called after a successful database connection has been established
Sub Software_MSSQL_Hook(AC, instance)
	On Error Resume Next
	Err.clear
	
	Dim dbRS: Set dbRS = CreateObject("ADODB.Recordset")
	Dim dbWho: Set dbWho = CreateObject("Scripting.Dictionary")
	
	' Connected hosts
	Err.clear
	Call dbRS.Open("EXEC sp_who", AC)
	If Err = 0 Then
		Do Until dbRS.EOF
		With dbRS.Fields
			Dim dbn: dbn = .Item("dbname")
			Dim dbh: dbh = .Item("hostname")
			
			' Host can be empty and needs to be trimmed
			If IsNull(dbh) Then: dbh = "": Else: dbh = Trim(dbh): End If
			
			' Empty hostnames are uninteresting
			If dbh <> "" Then
				' Add to list or append as comma-separated list if not already found
				If Not dbWho.Exists(dbn) Then
					Call dbWho.Add(dbn, dbh)
				ElseIf Not (InStr(dbWho(dbn), dbh) > 0) Then
					dbh = dbWho(dbn) & "," & dbh
					Call dbWho.Remove(dbn)
					Call dbWho.Add(dbn, dbh)
				End If
			End If
			
			dbRS.MoveNext
		End With
		Loop
		dbRS.Close
	End If
	
	
	' Databases
	Call dbRS.Open("EXEC sp_databases", AC)
	Do Until dbRS.EOF
		Dim strDB: strDB = dbRS.Fields.Item("DATABASE_NAME")
		
		' Make sure host is always set (none=blank)
		If Not dbWho.Exists(strDB) Then: Call dbWho.Add(strDB,""): End If

		' Call the MSSQL hooks
		Call Software_MSSQL_SharePoint(instance, strDB, AC, dbWho(strDb))
				
		dbRS.MoveNext
	Loop
	dbRS.Close
End Sub
'********************************************
Dim bSoftware_MSSQL: bSoftware_MSSQL = False

'Purpose:
'Gathers and writes information about Microsoft SQL Server
Sub Software_MSSQL(instance, state)
	On Error Resume Next
	Err.clear
	
	WScript.Echo "------- SQL Edition info from instance " & instance & " on " & strComputer & " [" & state & "]"
	
	Dim connect: connect = Replace(instance, "MSSQL$", "")
		
	' Creating CSV file for Services info
	' First time an SQL instance is detected a new file is created
	Dim File: Set File = CreateOrOpenFile("SQL.txt", bSoftware_MSSQL)
    if (FileIsEmpty("SQL.txt")) Then
        File.WriteLine CSV(Array("ComputerName", "State", "ConnectionMode", "InstanceName", "Edition", "Version"))	
    End If	
	
	If Not bSoftware_MSSQL Then: bSoftware_MSSQL = True: End If

	' All SQL instances must be registered
	File.WriteLine CSV(Array(strAliasComputerName, state, "Service", connect, "", ""))
	
	' Registry based version/edition detection
	Dim s,x,t,v: Call REG.EnumValues(HKLM, "SOFTWARE\Microsoft\Microsoft SQL Server\Instance Names\SQL",x,t)
	Dim vals(4)
	For Each s In x
		If UCase(s) = UCase(connect) Then
			Call REG.GetStringValue(HKLM, "SOFTWARE\Microsoft\Microsoft SQL Server\Instance Names\SQL",s,v)
			Call REG.GetStringValue(HKLM, "SOFTWARE\Microsoft\Microsoft SQL Server\" & v & "\Setup","Edition",vals(0))
			'Call REG.GetStringValue(HKLM, "SOFTWARE\Microsoft\Microsoft SQL Server\" & v & "\Setup","EditionType",vals(1))
			Call REG.GetStringValue(HKLM, "SOFTWARE\Microsoft\Microsoft SQL Server\" & v & "\Setup","Version",vals(2))
			
			If connect = "MSSQLSERVER" Then
				File.WriteLine CSV(Array(strAliasComputerName, state, "Registry", connect, vals(0), vals(2)))
			Else
				File.WriteLine CSV(Array(strAliasComputerName, state, "Registry", instance, vals(0), vals(2)))
			End If
		End If
	Next
	
	' Connect to MSSQL WMI service
	Call ConnectWMISQL()
	
	' Reset vals array
	vals(0) = "": vals(1) = "": vals(2) = "": vals(3) = "": vals(4) = ""
	
	' WMI based SQL version/edition detection
    Dim o,q: Set q = WMISQLQuery("Select * from SqlServiceAdvancedProperty where ServiceName Like '" & connect & "' or ServiceName Like 'MSSQL$" & connect & "'")
	For Each o In q
	With o
		Select Case UCase(.PropertyName)
			Case "SKUNAME":   vals(0) = .PropertyStrValue
			Case "CLUSTERED": vals(1) = .PropertyNumValue
			Case "VERSION":   vals(2) = .PropertyStrValue
			Case "SKU":       vals(3) = .PropertyNumValue
							  vals(4) = .ServiceName
		End Select
	End With
	Next
	
	' Write result of WMISQL query
	If q.Count > 0 Then
		If ""&vals(4) = "" Then: vals(4) = connect: End If
		File.WriteLine CSV(Array(strAliasComputerName, state, "WMI", vals(4), vals(0), vals(2)))
	End If
	
	' Login will fail if not running
	If State <> "Running" Then: Exit Sub: End If
	
	' Set connection string to Computer\Instance or just Computer for default instance (MSSQLSERVER)
	If connect = "MSSQLSERVER" Then
		instance = connect
		connect = strComputer
	Else
		connect = strComputer & "\" & connect
	End If
	
	' Attempt to connect with standard users/credentials
	If Not Software_MSSQL_Connect(connect, "", "", instance, File) Then
		If Not Software_MSSQL_Connect(connect, "sa", "", instance, File) Then
			If Not Software_MSSQL_Connect(connect, strUserSQL, strPasswordSQL, instance, File) Then
				' Connection could not be established
				File.WriteLine CSV(Array(strAliasComputerName, state, "Login", Replace(instance, "MSSQL$", ""), "Connection failed", ""))
			End If
		End If
	End If
End Sub

'********************************************
'Purpose:
'Execute a WMI query against all WMI SQL classes until one returns results
Function WMISQLQuery(strQuery)
    On Error Resume Next
    Err.clear

    Dim i

    ' Iterate through each SQL WMI class (One per major version)
    For i = 0 to UBound(WMISQL)
        Dim q: Set q = WMISQL(i).ExecQuery(strQuery)
	    If Not ((IsNull(q)) Or (IsEmpty(q))) Then
            If q.Count > 0 Then 
                Set WMISQLQuery = q
                Exit Function
            End If
        End If
    Next

End Function
'********************************************
'Purpose:
'Establishes a connection to a Microsoft SQL Server instance
Function Software_MSSQL_Connect(connect, user, password, instance, File)
	On Error Resume Next
	Err.clear
	
	' ADODB connection
	Dim AC: Set AC = CreateObject("ADODB.Connection")
	Dim RS: Set RS = CreateObject("ADODB.Recordset")
	
	' Build connect string depending on credentials
	If user = "" Then
		AC.ConnectionString = "Provider=SQLOLEDB;Data Source=" & connect & ";Integrated Security=SSPI;Connect Timeout = 30"
	Else
		If password = "" Then
			AC.ConnectionString = "Provider=SQLOLEDB.1;Persist Security Info=False;User ID=" & user & ";Data Source=" & connect
		Else
			AC.ConnectionString = "Provider=SQLOLEDB.1;Password=" & password & ";Persist Security Info=False;User ID=" & user & ";Data Source=" & connect
		End If
	End If
	
	' Attempt connection
	Err.clear
	Call AC.Open()
	If Err = 0 Then
		RS.Open "Select SERVERPROPERTY('Edition') AS [Edition],SERVERPROPERTY('InstanceName') AS [InstanceName],@@VERSION AS [Server Information],SERVERPROPERTY('productversion') AS [ProductVersion],SERVERPROPERTY('ProductLevel') AS [ProductLevel], SERVERPROPERTY('ISCLUSTERED') AS ISCLUSTERED", AC
		If Err = 0 Then
		With RS.Fields
			File.WriteLine CSV(Array(strAliasComputerName, "Running", "Login", instance, .Item("Edition"), .Item("ProductVersion")))
		End With
		End If
		
		Call Software_MSSQL_Hook(AC, instance)
		Call AC.Close()
		
		Software_MSSQL_Connect = True
	Else
		Software_MSSQL_Connect = False
	End If
End Function
'********************************************
'********************************************
'********************************************
'********************************************
'Purpose:
'Gather and write information about hosts and aliases in the AD forest based on DNS records
Function DNS
	On Error Resume Next
	Err.clear
	
	DNS = True
	
	Call ConnectWMIDNS()
	If WMIDNS Is Nothing Then: Exit Function: End If
	
	WScript.Echo "------- DNS records from " & strComputer & " -------"
	
    ' Creating CSV file for DNS records
	Dim File: Set File = CreateFile("DNS.txt")
    File.WriteLine CSV(Array("ComputerName", "OwnerName", "PrimaryName", "DomainName"))
    
	Dim strTopLevelDomain: strTopLevelDomain = strComputerDomain

	If InStrRev(strTopLevelDomain, ".") > 0 Then
		strTopLevelDomain = Left(strTopLevelDomain, InStrRev(strTopLevelDomain, ".")-1)
		If InStrRev(strTopLevelDomain, ".") > 0 Then
			strTopLevelDomain = Mid(strComputerDomain, InStrRev(strTopLevelDomain, ".")+1)
		Else
			strTopLevelDomain = strComputerDomain
		End If
	End If

	Dim o,q: Set q = WMIDNS.ExecQuery("Select * From MicrosoftDNS_AType")
	For Each o In q
	With o
		If Instr(UCase(.DomainName), UCase(strTopLevelDomain)) > 0 Then
			File.WriteLine CSV(Array(strAliasComputerName, .OwnerName, "", .DomainName))
		End If
	End With
	Next
	
	Set q = WMIDNS.ExecQuery("Select * From MicrosoftDNS_CNAMEType")
	For Each o In q
	With o
		If Instr(UCase(.DomainName), UCase(strTopLevelDomain)) > 0 Then
			File.WriteLine CSV(Array(strAliasComputerName, .OwnerName, .PrimaryName, .DomainName))
		End If
	End With
	Next
	
	File.Close
	DNS = True
End Function
'********************************************
'Purpose:
'Check if CPU is under high load
Function CPUload
	On Error Resume Next
	Err.clear
	
	Dim o,q: Set q = WMI.ExecQuery("Select deviceid, loadpercentage from Win32_Processor where DeviceID='CPU0'")
	CPUload = 42
	For Each o In q
		CPUload = o.LoadPercentage
		If IsNull(CPUload) Then: CPUload = 42: End If
	Next
End Function

' Script initialization
Call Initialize()
Call Execute()